#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

import click
from dxi._lib.util import boolean_based_system_exit
from dxi.database.dxi_database import DXIDatabase
from dxi.database.dxi_database import DXIDatabaseConstants


@click.group(
    short_help="\b\nPerform Database Operations "
               "on DSOURCE, VDB or VFILE.\n"
                "[ingest | provision | refresh | "
               "rewind | start | stop \nenable | disable | list]"
)
def database():
    """
    Perform Dsource, VDB and vFiles related operations.
    """
    pass


# Refresh command
@database.command()
@click.option(
    "--db-name",
    required=True,
    help="\b\nName of the virtual dataset to refresh.",
    default=None,
)
@click.option(
    "--timestamp-type",
    help="\b\nThe type of timestamp you are specifying for refresh.\n"
    "Options: [TIME | SNAPSHOT]\n"
    "[default: SNAPSHOT]",
    default="SNAPSHOT",
)
@click.option(
    "--timestamp",
    default="LATEST",
    help="""The Delphix semantic for the point in time on the source db or vfile 
    from which you want to refresh your VDB/vFile. \n
    Formats:                                                                      
    To use Latest Snapshot or Timepoint: LATEST                            
    To use a point in time: "YYYY-MM-DDTHH:MI:SS"                                           
    To use a snapshot name: "@YYYY-MM-DDTHH24:MI:SS.ZZZ"        
    To use snapshot time from GUI : "YYYY-MM-DDTHH:MI:SS"
    [default: LATEST]
    
    * A timestamp value should be in 24 hour format.     
    * Provided values should be enclosed in double quotes.                                                                
    """,
)
@click.option(
    "--timeflow",
    help="Name of the source database timeflow to refresh the virtual dataset.",
    default=None,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def refresh(
    db_name,
    timestamp_type,
    timestamp,
    engine,
    single_thread,
    timeflow,
    poll,
    config,
    log_file_path,
    parallel,
):
    """
    Refresh a Delphix VDB or vFile.
    """

    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )

    boolean_based_system_exit(
        obj.refresh(
            db_name=db_name,
            timestamp_type=timestamp_type,
            timestamp=timestamp,
            timeflow=timeflow,
        )
    )


# Rewind Command
@database.command()
@click.option(
    "--db-name",
    required=True,
    help="Name of the virtual dataset to rewind.",
    default=None,
)
@click.option(
    "--timestamp-type",
    help="\b\nThe type of timestamp you are specifying for refresh.\n"
    "Options: [TIME | SNAPSHOT]\n"
    "[default: SNAPSHOT]",
    default="SNAPSHOT",
)
@click.option(
    "--timestamp",
    default="LATEST",
    help="""The Delphix semantic for the point in time on the source db or vfile 
    from which you want to refresh your VDB/vFile. \n
    Formats:                                                                      
    To use Latest Snapshot or Timepoint: LATEST                            
    To use a point in time: "YYYY-MM-DDTHH:MI:SS"                                           
    To use a snapshot name: "@YYYY-MM-DDTHH24:MI:SS.ZZZ"        
    To use snapshot time from GUI : "YYYY-MM-DDTHH:MI:SS"
    [default: LATEST]
    
    * A timestamp value should be in 24 hour format.     
    * Provided values should be enclosed in double quotes.                                                                
    """,
)
@click.option(
    "--db-type",
    help="Type of database: oracle, mssql, ase, vfiles",
    default=None,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def rewind(
    db_name,
    timestamp_type,
    timestamp,
    engine,
    single_thread,
    db_type,
    poll,
    config,
    log_file_path,
    parallel,
):
    """
    Rewind a VDB or vFile.
    """

    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )
    boolean_based_system_exit(
        obj.rewind(
            db_name,
            timestamp_type=timestamp_type,
            timestamp=timestamp,
            db_type=db_type,
        )
    )


# Delete command
@database.command()
@click.option(
    "--db-name",
    default=None,
    required=True,
    help="\b\nName of datasets to delete.\n"
    "To delete multiple datasets, separate them with ':'.\n"
    "Usage: vdb1:vdb2",
)
@click.option(
    "--db-type",
    default=DXIDatabaseConstants.TYPE,
    help="\b\nType of the dataset to delete.\n"
    "[vdb | dsource]\n"
    "[default: vdb]",
)
@click.option(
    "--force",
    is_flag=True,
    default=DXIDatabaseConstants.FORCE,
    help="Force delete the dataset(s).",
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def delete(
    db_name,
    db_type,
    engine,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
    force,
):
    """
    Delete a Delphix dSource or VDB.
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )
    boolean_based_system_exit(
        obj.delete(db_name, db_type=db_type, force=force)
    )


#############################################
# DB Operations

# db-list
@database.command()
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def list(engine, single_thread, parallel, poll, config, log_file_path):
    """
    List all datasets on an engine.
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )
    boolean_based_system_exit(obj.list())


# db-start
@database.command()
@click.option(
    "--db-name",
    required=True,
    help="Name of the virtual dataset to start.",
    default=None,
)
@click.option("--group", help="Group where the dataset resides.", default=None)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def start(
    db_name,
    group,
    engine,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
):
    """
    Starts a virtual dataset by name and group
    """
    ops_obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        parallel=parallel,
        log_file_path=log_file_path,
    )
    boolean_based_system_exit(ops_obj.start(db_name=db_name, group=group))


# db-stop
@database.command()
@click.option(
    "--db-name",
    required=True,
    help="Name of the virtual dataset to stop.",
    default=None,
)
@click.option("--group", help="Group where the dataset resides", default=None)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def stop(
    db_name,
    group,
    engine,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
):
    """
    Stop a virtual dataset by name and group (optional)
    """
    ops_obj = DXIDatabase(
        engine=engine,
        poll=poll,
        config=config,
        parallel=parallel,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(ops_obj.stop(db_name=db_name, group=group))


# db-enable
@database.command()
@click.option(
    "--db-name",
    required=True,
    help="Name of the virtual dataset to enable.",
    default=None,
)
@click.option("--group", help="Group where the dataset resides.", default=None)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def enable(
    db_name,
    group,
    engine,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
):
    """
    Enable a virtual dataset by name and group(optional)
    """
    ops_obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        parallel=parallel,
        log_file_path=log_file_path,
    )
    boolean_based_system_exit(ops_obj.enable(db_name=db_name, group=group))


# db-disable
@database.command()
@click.option(
    "--db-name",
    required=True,
    help="Name of the virtual dataset to disable.",
    default=None,
)
@click.option("--group", help="Group where the dataset resides.", default=None)
@click.option(
    "--force",
    is_flag=True,
    help="Force disable a virtual dataset.",
    default=None,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def disable(
    db_name,
    group,
    force,
    engine,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
):
    """
    Disable a virtual dataset by name and group(optional).
    """
    ops_obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )
    boolean_based_system_exit(
        ops_obj.disable(db_name=db_name, group=group, force=force)
    )


# Create dSource
@click.group()
def ingest():
    """
    Ingest/Link a source database into Delphix.
    """
    pass


@ingest.command()
@click.option(
    "--source-name", required=True, help="Name of the dSource to create."
)
@click.option("--db-user", help="Source DB username.", required=True)
@click.option(
    "--db-password", help="Source DB user password.", required=True
)
@click.option("--group", help="Group name for the dsource.", required=True)
@click.option(
    "--env-name",
    help="Name of the host environment for the dsource.",
    required=True,
)
@click.option(
    "--env-inst",
    help="\b\nFull path of the database binary on the source host.\n"
    "For oracle, this is the path to Oracle db_home.",
    required=True,
)
@click.option(
    "--port-num",
    help="Port number for the oracle listener.",
    required=True,
    default=DXIDatabaseConstants.ORACLE_DEFAULT_PORT,
)
@click.option(
    "--logsync", help="Enable or disable logsync.", is_flag=True, default=False
)
@click.option("--ip-addr", help="IP Address of the dsource.")
@click.option(
    "--rman-channels",
    help="Configures the number of Oracle RMAN Channels.",
    default=2,
)
@click.option(
    "--files-per-set",
    help="Configures how many files per set for Oracle RMAN.",
    default=5,
)
@click.option(
    "--num-connections",
    help="Number of connections for Oracle RMAN.",
    default=5,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def oracle(
    engine,
    ip_addr,
    env_name,
    group,
    source_name,
    db_user,
    db_password,
    env_inst,
    num_connections,
    log_file_path,
    logsync,
    single_thread,
    files_per_set,
    rman_channels,
    port_num,
    parallel,
    poll,
    config,
):
    """
    Ingest/Link an Oracle dSource.
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        parallel=parallel,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
    )
    boolean_based_system_exit(
        obj.ingest_oracle(
            source_name,
            db_password,
            db_user,
            group,
            env_inst,
            ip_addr=ip_addr,
            env_name=env_name,
            num_connections=num_connections,
            logsync=logsync,
            files_per_set=files_per_set,
            rman_channels=rman_channels,
            port_num=port_num,
        )
    )


@ingest.command()
@click.option(
    "--source-name", required=True, help="Name of the dSource to create."
)
@click.option("--db-user", help="Source DB user.", required=True)
@click.option("--db-password", help="Source DB user password.", required=True)
@click.option("--group", help="Group name for this dSource.", required=True)
@click.option("--logsync", help="Enable or disable logsync", is_flag=True, default=False)
@click.option(
    "--source-env",
    help="Name of the environment to use as source.",
    required=True,
)
@click.option(
    "--stage-env",
    help="Name of the environment to use as staging.",
    required=True,
)
@click.option(
    "--stage-instance",
    help="Name of the staging instance for ingestion.",
    required=True,
)
@click.option(
    "--backup-user",
    help="User with access to the shared location for backups.",
    default=None
)
@click.option(
    "--backup-user-pwd",
    help="Password of the shared backup path.",
    default=None,
)
@click.option(
    "--validated-sync-mode",
    help="Delphix will try to load the most recent backup.",
    default="TRANSACTION_LOG",
)
@click.option(
    "--delphix-managed",
    help="If set, Delphix will manage the backups.",
    default=False,
    is_flag=True,
)
@click.option(
    "--initial-load-type",
    help="Delphix will try to load the most recent backup.",
    default=None,
)
@click.option(
    "--backup-path", help="Full path to the MS SQLServer backups", default=None
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def mssql(
    engine,
    group,
    source_name,
    db_user,
    db_password,
    log_file_path,
    logsync,
    single_thread,
    parallel,
    poll,
    config,
    stage_instance,
    stage_env,
    backup_user_pwd,
    backup_user,
    initial_load_type,
    validated_sync_mode,
    delphix_managed,
    backup_path,
    source_env
):
    """
    Link an MSSQL dSource
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        parallel=parallel,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
    )
    boolean_based_system_exit(
        obj.ingest_mssql(
            group=group,
            source_name=source_name,
            db_user=db_user,
            db_password=db_password,
            logsync=logsync,
            stage_instance=stage_instance,
            source_env=source_env,
            stage_env=stage_env,
            backup_user_pwd=backup_user_pwd,
            backup_user=backup_user,
            initial_load_type=initial_load_type,
            validated_sync_mode=validated_sync_mode,
            delphix_managed=delphix_managed,
            backup_path=backup_path,
        )
    )


@ingest.command()
@click.option(
    "--source-name", required=True, help="Name of the dSource to create."
)
@click.option("--db-user", help="Source DB user.", required=True)
@click.option("--db-password", help="Sourde DB user password.", required=True)
@click.option("--group", help="Group name for this dSource.", required=True)
@click.option("--env-name", help="Staging environment name", required=True)
@click.option("--source-env", help="Source environment name", required=True)
@click.option(
    "--stage-instance",
    help="The SAP ASE instance on the staging environment to use for sync.",
    required=True,
)
@click.option(
    "--backup-path",
    help="Path to the Sybase backups.",
    default=None,
    required=True,
)
@click.option("--logsync", help="Enable or disable logsync.", is_flag=True, default=False)
@click.option(
    "--backup-files",
    help="Backup file list (add space between multiple files)",
    default=None,
)
@click.option(
    "--create-backup",
    help="The parameters to use as input to sync a SAP ASE database by taking a new full backup.",
    default=False,
    is_flag=True,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def sybase(
    engine,
    env_name,
    group,
    source_name,
    db_user,
    db_password,
    log_file_path,
    logsync,
    single_thread,
    parallel,
    poll,
    config,
    stage_instance,
    backup_path,
    backup_files,
    create_backup,
    source_env,
):
    """
    Ingest an ASE/Sybase dsource.
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        parallel=parallel,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
    )
    boolean_based_system_exit(
        obj.ingest_sybase(
            source_name=source_name,
            db_user=db_user,
            db_password=db_password,
            group=group,
            source_env=source_env,
            stage_env=env_name,
            stage_instance=stage_instance,
            backup_path=backup_path,
            logsync=logsync,
            backup_files=backup_files,
            create_backup=create_backup,
        )
    )


database.add_command(ingest)
"""
Methods to provision different database types.
"""


@click.group()
def provision():
    """
    Provision Delphix Virtual Databases.
    """
    pass


@provision.command()
@click.option("--db-name", required=True, help="Name of the VDB.")
@click.option("--source-name", required=True, help="The source database name.")
@click.option(
    "--group",
    required=True,
    help="The group into which Delphix will place the VDB.",
)
@click.option(
    "--port-num",
    help="The port number of the database instance.",
    required=True,
)
@click.option(
    "--env-name",
    required=True,
    help="The name of the Target environment in Delphix",
)
@click.option(
    "--env-inst",
    help="\b\nThe identifier for ASE instance on the target environment.\n"
    "This is the ASE instance where the VDB will be provisioned. ",
    required=True,
)
@click.option(
    "--mntpoint",
    help="Mount folder that Delphix can use on the target host.",
    required=True,
)
@click.option(
    "--timestamp-type",
    help="\b\nThe type of timestamp you are specifying.\n"
    "[TIME | SNAPSHOT]\n"
    "[Default: default]",
    default="SNAPSHOT",
)
@click.option(
    "--timestamp",
    help="""The Delphix semantic for the point in time on the source db or vfile 
    from which you want to refresh your VDB/vFile. \n
    Formats:                                                                      
    To use Latest Snapshot or Timepoint: LATEST                            
    To use a point in time:"YYYY-MM-DDTHH:MI:SS"                                           
    To use a snapshot name:"@YYYY-MM-DDTHH24:MI:SS.ZZZ"                                           
    To use snapshot time from GUI:"YYYY-MM-DDTHH:MI:SS"                                           
    [default: LATEST]                                           
    
    * A timestamp value should be in 24 hour format.     
    * Provided values should be enclosed in double quotes.                                                                
    """,
    default="LATEST",
)
@click.option(
    "--prerefresh",
    help="Hook name to run before a refresh only (not during initial provision). ",
    default=False,
)
@click.option(
    "--postrefresh",
    help="Hook name to run after a refresh only (runs after Configure Clone).",
    default=False,
)
@click.option(
    "--prerollback", help="Hook name to run before a rewind", default=False
)
@click.option(
    "--postrollback", help="Hook name to run after a rewind", default=False
)
@click.option(
    "--configure-clone", help="Configure Clone commands", default=False
)
@click.option(
    "--no-truncate-log",
    help="Set the trunc log on chkpt database option.",
    default=False,
    is_flag=True,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def sybase(
    engine,
    source_name,
    db_name,
    group,
    mntpoint,
    env_name,
    timestamp,
    timestamp_type,
    prerefresh,
    postrefresh,
    prerollback,
    postrollback,
    configure_clone,
    env_inst,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
    port_num,
    no_truncate_log,
):
    """
    Create a Sybase ASE VDB
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )
    boolean_based_system_exit(
        obj.provision_ase(
            group,
            source_name,
            db_name,
            env_name,
            timestamp=timestamp,
            timestamp_type=timestamp_type,
            prerefresh=prerefresh,
            postrefresh=postrefresh,
            prerollback=prerollback,
            postrollback=postrollback,
            configure_clone=configure_clone,
            env_inst=env_inst,
            port_num=port_num,
            mntpoint=mntpoint,
            no_truncate_log=no_truncate_log,
        )
    )


@provision.command()
@click.option(
    "--source-name", required=True, help="Name of the source database."
)
@click.option("--db-name", required=True, help="Name for the VDB.")
@click.option(
    "--env-name",
    required=True,
    help="The name of the target environment in Delphix.",
)
@click.option(
    "--env-inst",
    help="\b\nThe identifier for MS SQLServer instance on the target environment.\n"
    "This is the SQLServer instance where the VDB will be provisioned. ",
    required=True,
)
@click.option(
    "--mntpoint",
    help="Mount folder that Delphix can use on the target host.",
    required=True,
)
@click.option(
    "--group",
    help="The group into which Delphix will place the VDB.",
    required=True,
)
@click.option("--port-num", help="The port number of the database instance.")
@click.option(
    "--timestamp-type",
    help="\b\nThe type of timestamp you are specifying.\n"
    "[TIME | SNAPSHOT]\n"
    "[Default: default]",
    default="SNAPSHOT",
)
@click.option(
    "--timestamp",
    help="""The Delphix semantic for the point in time on the source db or vfile 
    from which you want to refresh your VDB/vFile. \n
    Formats:                                                                      
    To use Latest Snapshot or Timepoint: LATEST                            
    To use a point in time:"YYYY-MM-DDTHH:MI:SS"                                           
    To use a snapshot name:"@YYYY-MM-DDTHH24:MI:SS.ZZZ"                                           
    To use snapshot time from GUI:"YYYY-MM-DDTHH:MI:SS"                                           
    [default: LATEST]      
    
    * A timestamp value should be in 24 hour format.     
    * Provided values should be enclosed in double quotes.                                                                
    """,
    default="LATEST",
)
@click.option(
    "--prerefresh",
    help="Hook name to run before a refresh only (not during initial provision). ",
    default=None,
)
@click.option(
    "--postrefresh",
    help="Hook name to run after a refresh only (runs after Configure Clone).",
    default=None,
)
@click.option(
    "--prerollback", help="Hook name to run before a rewind", default=None
)
@click.option(
    "--postrollback", help="Hook name to run after a rewind.", default=None
)
@click.option(
    "--configure-clone",
    help="Hook name to run after initial provision and after a refresh.",
    default=None,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def mssql(
    engine,
    source_name,
    db_name,
    group,
    mntpoint,
    env_name,
    timestamp,
    timestamp_type,
    prerefresh,
    postrefresh,
    prerollback,
    postrollback,
    configure_clone,
    env_inst,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
    port_num,
):
    """
    Provision a MS SQLServer VDB.
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )
    boolean_based_system_exit(
        obj.provision_mssql(
            group,
            source_name,
            db_name,
            env_name,
            timestamp=timestamp,
            timestamp_type=timestamp_type,
            prerefresh=prerefresh,
            postrefresh=postrefresh,
            prerollback=prerollback,
            postrollback=postrollback,
            configure_clone=configure_clone,
            env_inst=env_inst,
            port_num=port_num,
            mntpoint=mntpoint,
        )
    )


@provision.command()
@click.option("--source-name", required=True, help="The source database name.")
@click.option("--db-name", required=True, help="Name for the VDB.")
@click.option(
    "--env-name",
    required=True,
    help="The name of the Target environment in Delphix",
)
@click.option(
    "--group",
    required=True,
    help="The group into which Delphix will place the VDB",
)
@click.option(
    "--env-inst",
    help="The identifier of the instance in Delphix.",
    required=True,
)
@click.option(
    "--mntpoint",
    help="Mount folder that Delphix can use on the target host.",
    required=True,
)
@click.option("--port-num", help="The port number of the database instance.")
@click.option(
    "--timestamp-type",
    help="\b\nThe type of timestamp you are specifying.\n" "[TIME | SNAPSHOT]",
    default="SNAPSHOT",
)
@click.option(
    "--timestamp",
    help="""The Delphix semantic for the point in time on the source db or vfile 
    from which you want to refresh your VDB/vFile. \n
    Formats:                                                                      
    To use Latest Snapshot or Timepoint: LATEST                            
    To use a point in time:"YYYY-MM-DDTHH:MI:SS"                                           
    To use a snapshot name:"@YYYY-MM-DDTHH24:MI:SS.ZZZ"                                           
    To use snapshot time from GUI:"YYYY-MM-DDTHH:MI:SS"                                           
    [default: LATEST]      
    
    * A timestamp value should be in 24 hour format.     
    * Provided values should be enclosed in double quotes.                                                                
    """,
    default="LATEST",
)
@click.option(
    "--prerefresh",
    help="Hook name to run before a refresh only (not during initial provision). ",
    default=None,
)
@click.option(
    "--postrefresh",
    help="Hook name to run after a refresh only (runs after Configure Clone).",
    default=None,
)
@click.option(
    "--prerollback", help="Hook name to run before a rewind", default=None
)
@click.option(
    "--postrollback", help="Hook name to run after a rewind", default=None
)
@click.option(
    "--configure-clone",
    help="Hook name to run after initial provision and after a refresh.",
    default=None,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def oracle_mt(
    engine,
    source_name,
    db_name,
    group,
    mntpoint,
    env_name,
    timestamp,
    timestamp_type,
    prerefresh,
    postrefresh,
    prerollback,
    postrollback,
    configure_clone,
    env_inst,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
    port_num,
):
    """
    Create an Oracle Multi Tenant VDB
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )
    boolean_based_system_exit(
        obj.provision_oracle_mt(
            group,
            source_name,
            db_name,
            env_name,
            timestamp=timestamp,
            timestamp_type=timestamp_type,
            prerefresh=prerefresh,
            postrefresh=postrefresh,
            prerollback=prerollback,
            postrollback=postrollback,
            configure_clone=configure_clone,
            env_inst=env_inst,
            port_num=port_num,
            mntpoint=mntpoint,
        )
    )


@provision.command()
@click.option("--source-name", required=True, help="The source database")
@click.option("--db-name", required=True, help="Name of the VDB.")
@click.option(
    "--group",
    required=True,
    help="The group into which Delphix will place the VDB",
)
@click.option(
    "--env-name",
    required=True,
    help="The name of the Target environment in Delphix",
)
@click.option(
    "--env-inst",
    help="\b\nFull path of the database binary on the target host.",
    required=True,
)
@click.option(
    "--mntpoint",
    help="Mount folder that Delphix can use on the target host.",
    required=True,
)
@click.option(
    "--port-num",
    help="\b\nThe port number of the database instance.\n" "[default: 1521]",
    default=DXIDatabaseConstants.ORACLE_DEFAULT_PORT,
)
@click.option(
    "--timestamp-type",
    help="\b\nThe type of timestamp you are specifying.\n" "[TIME | SNAPSHOT]",
    default="SNAPSHOT",
)
@click.option(
    "--timestamp",
    help="""The Delphix semantic for the point in time on the source db or vfile 
    from which you want to refresh your VDB/vFile. \n
    Formats:                                                                      
    To use Latest Snapshot or Timepoint: LATEST                            
    To use a point in time:"YYYY-MM-DDTHH:MI:SS"                                           
    To use a snapshot name:"@YYYY-MM-DDTHH24:MI:SS.ZZZ"                                           
    To use snapshot time from GUI:"YYYY-MM-DDTHH:MI:SS"                                           
    [default: LATEST]      
    
    * A timestamp value should be in 24 hour format.     
    * Provided values should be enclosed in double quotes.                                                                
    """,
    default="LATEST",
)
@click.option(
    "--prerefresh",
    help="Hook name to run before a refresh only (not during initial provision). ",
    default=None,
)
@click.option(
    "--postrefresh",
    help="Hook name to run after a refresh only (runs after Configure Clone).",
    default=None,
)
@click.option(
    "--prerollback", help="Hook name to run before a rewind", default=None
)
@click.option(
    "--postrollback", help="Hook name to run after a rewind", default=None
)
@click.option(
    "--configure-clone",
    help="Hook name to run after initial provision and after a refresh.",
    default=None,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def oracle_si(
    engine,
    source_name,
    db_name,
    group,
    mntpoint,
    env_name,
    timestamp,
    timestamp_type,
    prerefresh,
    postrefresh,
    prerollback,
    postrollback,
    configure_clone,
    env_inst,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
    port_num,
):
    """
    Create an Oracle SI VDB
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )
    boolean_based_system_exit(
        obj.provision_oracle_si(
            group,
            source_name,
            db_name,
            env_name,
            timestamp=timestamp,
            timestamp_type=timestamp_type,
            prerefresh=prerefresh,
            postrefresh=postrefresh,
            prerollback=prerollback,
            postrollback=postrollback,
            configure_clone=configure_clone,
            env_inst=env_inst,
            port_num=port_num,
            mntpoint=mntpoint,
        )
    )


@provision.command()
@click.option("--db-name", required=True, help="Name for the VDB/vFile.")
@click.option("--source-name", required=True, help="The source database.")
@click.option(
    "--group",
    required=True,
    help="The group into which Delphix will place the VDB.",
)
@click.option(
    "--env-name",
    required=True,
    help="The name of the target environment in Delphix.",
)
@click.option(
    "--env-inst",
    help="\b\nFull path of the database binary on the target host.",
    required=True,
)
@click.option("--port-num", help="The port number of the database instance.")
@click.option(
    "--timestamp-type",
    help="The type of timestamp you are specifying. TIME or SNAPSHOT",
    default="SNAPSHOT",
)
@click.option(
    "--timestamp",
    help="""The Delphix semantic for the point in time on the source db or vfile 
    from which you want to refresh your VDB/vFile. \n
    Formats:                                                                      
    To use Latest Snapshot or Timepoint: LATEST                            
    To use a point in time:"YYYY-MM-DDTHH:MI:SS"                                           
    To use a snapshot name:"@YYYY-MM-DDTHH24:MI:SS.ZZZ"                                           
    To use snapshot time from GUI:"YYYY-MM-DDTHH:MI:SS"                                           
    [default: LATEST]      
    
    * A timestamp value should be in 24 hour format.     
    * Provided values should be enclosed in double quotes.                                                                
    """,
    default="LATEST",
)
@click.option(
    "--mntpoint",
    help="Mount folder that Delphix can use on the target host.",
    default="/mnt/ingest",
)
@click.option(
    "--prerefresh",
    help="Hook name to run before a refresh only (not during initial provision). ",
    default=None,
)
@click.option(
    "--postrefresh",
    help="Hook name to run after a refresh only (runs after Configure Clone).",
    default=None,
)
@click.option(
    "--prerollback", help="Hook name to run before a rewind", default=None
)
@click.option(
    "--postrollback", help="Hook name to run after a rewind", default=None
)
@click.option(
    "--configure-clone",
    help="Hook name to run after initial provision and after a refresh.",
    default=None,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def postgres(
    engine,
    source_name,
    db_name,
    group,
    mntpoint,
    env_name,
    timestamp,
    timestamp_type,
    prerefresh,
    postrefresh,
    prerollback,
    postrollback,
    configure_clone,
    env_inst,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
    port_num,
):
    """
    Provision a Postgres VDB.
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )
    boolean_based_system_exit(
        obj.provision_postgres(
            group,
            source_name,
            db_name,
            env_name,
            timestamp=timestamp,
            timestamp_type=timestamp_type,
            prerefresh=prerefresh,
            postrefresh=postrefresh,
            prerollback=prerollback,
            postrollback=postrollback,
            configure_clone=configure_clone,
            env_inst=env_inst,
            port_num=port_num,
            mntpoint=mntpoint,
        )
    )


@provision.command()
@click.option("--db-name", required=True, help="Name for the VDB.")
@click.option("--source-name", required=True, help="The source database")
@click.option(
    "--env-name",
    required=True,
    help="The name of the target environment for the VDB.",
)
@click.option(
    "--env-inst",
    help="\b\nFull path of the database binary on the target host.",
    required=True,
)
@click.option(
    "--group",
    required=True,
    help="The group into which Delphix will place the VDB.",
)
@click.option("--port-num", help="The port number of the database instance.")
@click.option(
    "--timestamp-type",
    help="\b\nThe type of timestamp you are specifying.\n" "[TIME | SNAPSHOT]",
    default="SNAPSHOT",
)
@click.option(
    "--timestamp",
    help="""The Delphix semantic for the point in time on the source db or vfile 
    from which you want to refresh your VDB/vFile. \n
    Formats:                                                                      
    To use Latest Snapshot or Timepoint: LATEST                            
    To use a point in time:"YYYY-MM-DDTHH:MI:SS"                                           
    To use a snapshot name:"@YYYY-MM-DDTHH24:MI:SS.ZZZ"                                           
    To use snapshot time from GUI:"YYYY-MM-DDTHH:MI:SS"                                           
    [default: LATEST]      
    
    * A timestamp value should be in 24 hour format.     
    * Provided values should be enclosed in double quotes.                                                                
    """,
    default="LATEST",
)
@click.option(
    "--mntpoint",
    help="Mount folder that Delphix can use on the target host.",
    default="/mnt/ingest",
)
@click.option(
    "--prerefresh",
    help="Hook name to run before a refresh only (not during initial provision). ",
    default=None,
)
@click.option(
    "--postrefresh",
    help="Hook name to run after a refresh only (runs after Configure Clone).",
    default=None,
)
@click.option(
    "--prerollback", help="Hook name to run before a rewind", default=None
)
@click.option(
    "--postrollback", help="Hook name to run after a rewind", default=None
)
@click.option(
    "--configure-clone",
    help="Hook name to run after initial provision and after a refresh.",
    default=None,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def vfiles(
    engine,
    source_name,
    db_name,
    group,
    mntpoint,
    env_name,
    timestamp,
    timestamp_type,
    prerefresh,
    postrefresh,
    prerollback,
    postrollback,
    configure_clone,
    env_inst,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
    port_num,
):
    """
    Create a vfiles VDB.
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )
    boolean_based_system_exit(
        obj.provision_vfiles(
            group,
            source_name,
            db_name,
            env_name,
            timestamp=timestamp,
            timestamp_type=timestamp_type,
            prerefresh=prerefresh,
            postrefresh=postrefresh,
            prerollback=prerollback,
            postrollback=postrollback,
            configure_clone=configure_clone,
            env_inst=env_inst,
            port_num=port_num,
            mntpoint=mntpoint,
        )
    )


# provision oracle rac
@provision.command()
@click.option(
    "--group",
    required=True,
    help="The group into which Delphix will place the VDB",
)
@click.option("--source-name", required=True, help="The source database")
@click.option(
    "--db-name", required=True, help="The name you want to give the database"
)
@click.option(
    "--env-name",
    required=True,
    help="The name of the Target environment in Delphix",
)
@click.option(
    "--nodes", required=True, help="Colon separated list of nodes in RAC"
)
@click.option("--port-num", help="The port number of the database instance")
@click.option(
    "--env-inst",
    help="The identifier of the instance in Delphix.",
    default=None,
)
@click.option(
    "--timestamp-type",
    help="The type of timestamp you are specifying. TIME or SNAPSHOT",
    default="SNAPSHOT",
)
@click.option(
    "--timestamp",
    help="\bThe Delphix semantic for the point in time\n "
         "from which you want to ingest your VDB.",
    default="LATEST",
)
@click.option(
    "--mntpoint",
    help="The identifier of the instance in Delphix.",
    default="/mnt/provision",
)
@click.option(
    "--prerefresh",
    help="Hook name to run before a refresh only (not during initial provision). ",
    default=None,
)
@click.option(
    "--postrefresh",
    help="Hook name to run after a refresh only (runs after Configure Clone).",
    default=None,
)
@click.option(
    "--prerollback", help="Hook name to run before a rewind", default=None
)
@click.option(
    "--postrollback", help="Hook name to run after a rewind", default=None
)
@click.option(
    "--configure-clone",
    help="Hook name to run after initial provision and after a refresh.",
    default=None,
)
@click.option(
    "--engine",
    default=DXIDatabaseConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(DXIDatabaseConstants.SINGLE_THREAD),
    default=DXIDatabaseConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIDatabaseConstants.POLL),
    default=DXIDatabaseConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=DXIDatabaseConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=DXIDatabaseConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=DXIDatabaseConstants.LOG_FILE_PATH,
)
def oracle_rac(
    engine,
    source_name,
    db_name,
    group,
    mntpoint,
    env_name,
    nodes,
    timestamp,
    timestamp_type,
    prerefresh,
    postrefresh,
    prerollback,
    postrollback,
    configure_clone,
    env_inst,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
    port_num,
):
    """
    Create an Oracle RAC VDB
    """
    obj = DXIDatabase(
        engine=engine,
        single_thread=single_thread,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        parallel=parallel,
    )
    obj.provision_oracle_rac(
        group,
        source_name,
        db_name,
        env_name,
        nodes=nodes,
        timestamp=timestamp,
        timestamp_type=timestamp_type,
        prerefresh=prerefresh,
        postrefresh=postrefresh,
        prerollback=prerollback,
        postrollback=postrollback,
        configure_clone=configure_clone,
        env_inst=env_inst,
        port_num=port_num,
        mntpoint=mntpoint,
    )


database.add_command(provision)

if __name__ == "__main__":
    print(mssql())



def run_async(func):
    """
    http://code.activestate.com/recipes/576684-simple-threading-decorator/
    run_async(func)
    function decorator, intended to make "func" run in a separate
    thread (asynchronously).
    Returns the created Thread object

    E.g.:
    @run_async
    def task1():
        do_something

    @run_async
    def task2():
        do_something_too

        t1 = task1()
        t2 = task2()
        ...
        t1.join()
        t2.join()
    """
    from threading import Thread
    from functools import wraps

    @wraps(func)
    def async_func(*args, **kwargs):
        func_hl = Thread(target=func, args=args, kwargs=kwargs)
        func_hl.start()
        return func_hl

    return async_func



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#
from enum import Enum

class DataLayoutType(Enum):
    DATA_TEMPLATE = "template"
    DATA_CONTAINER = "container"

class EnvironmentTypes(Enum):
    UNIX = "unix"
    WIN = "windows"
    LINUX = "linux"

class EnvironmentHostTypes(Enum):
    WIN = "WindowsHostEnvironment"
    WINCLUSTER = "WindowsCluster"

class HostTypes(Enum):
    WIN = "WindowsHost"
    UNIX = "UnixHost"

class EnvironmentOps(Enum):
    ADD = "add"
    UPDATEHOST = "updatehost"
    DELETE = "delete"
    REFRESH = "refresh"
    ENABLE = "enable"
    DISABLE = "disable"
    LIST = "list"

class BranchOps(Enum):
    CREATE = "create"
    DELETE = "delete"
    ACTIVATE = "activate"
    LIST = "list"

class BookmarkOps(Enum):
    CREATE = "create"
    DELETE = "delete"
    UPDATE = "update"
    SHARE = "share"
    UNSHARE = "unshare"
    LIST = "list"

class VirtualOps(Enum):
    PROVISION = "create"
    REFRESH = "refresh"
    REWIND = "rewind"
    SNAPSHOT = "snapshot"
    START = "start"
    STOP = "stop"
    ENABLE = "enable"
    DISABLE = "disable"
    DELETE = "delete"
    LIST = "list"

class SourceOps(Enum):
    CREATE = "create"
    DELETE = "delete"
    LINK = "link"
    UNLIK = "unlink"

class TemplateOps(Enum):
    CREATE = "create"
    DELETE = "delete"
    LIST = "list"

# Timestamp Formats
FORMAT_TS_MILLIS = '%Y-%m-%dT%H:%M:%S.%fZ'
FORMAT_TS_SEC = '%Y-%m-%dT%H:%M:%S'

# Default Ports
PORT_ORACLE_DEFAULT="1521"
PORT_MSSQL_DEFAULT="1433"
PORT_POSTGRES_DEFAULT = "5432"
PORT_SYBASE_DEFAULT = "5000"



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

import click
from dxi._lib.util import boolean_based_system_exit
from dxi.template.dxi_template import DXITemplate
from dxi.template.dxi_template import DXITemplateConstants


@click.group(
    short_help="\b\nPerform Delphix Self-Service Template Operations.\n"
               "[create | delete | list] "
)
def template():
    """
    Self-Service Template operations
    """
    pass


# List Command
@template.command()
@click.option(
    "--engine",
    default=DXITemplateConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXITemplateConstants.SINGLE_THREAD),
    default=DXITemplateConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file (including filename).",
    default=DXITemplateConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path of the logs folder.",
    default=DXITemplateConstants.LOG_FILE_PATH,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXITemplateConstants.POLL),
    default=DXITemplateConstants.POLL,
)
def list(engine, single_thread, config, log_file_path, poll):
    """
    List all self-service templates on a given engine.
    """
    temp_obj = DXITemplate(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(temp_obj.list())


# Create Template Command
@template.command()
@click.option(
    "--template-name", required=True, help=" Name of the template to create"
)
@click.option(
    "--db-name",
    required=True,
    help="\b\nList of data sources to add to the new template.\n"
    "If the template should contain multiple data source,\n"
    "separate the names with colon (:).\n"
    "Sample: db1:db2:db3",
)
@click.option(
    "--engine",
    default=DXITemplateConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXITemplateConstants.SINGLE_THREAD),
    default=DXITemplateConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file (including filename).",
    default=DXITemplateConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path of the logs folder.",
    default=DXITemplateConstants.LOG_FILE_PATH,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXITemplateConstants.POLL),
    default=DXITemplateConstants.POLL,
)
def create(
    template_name, db_name, engine, single_thread, config, log_file_path, poll
):
    """
    Create a self-service template.
    """
    temp_obj = DXITemplate(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(
        temp_obj.create(template_name=template_name, dbnames=db_name)
    )


# Delete Template Command
@template.command()
@click.option(
    "--template-name", required=True, help="Name of self-service template."
)
@click.option(
    "--engine",
    default=DXITemplateConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXITemplateConstants.SINGLE_THREAD),
    default=DXITemplateConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file (including filename).",
    default=DXITemplateConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path of the logs folder.",
    default=DXITemplateConstants.LOG_FILE_PATH,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXITemplateConstants.POLL),
    default=DXITemplateConstants.POLL,
)
def delete(template_name, engine, single_thread, config, log_file_path, poll):
    """
    Delete a self-service template.
    """
    ss_container = DXITemplate(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(ss_container.delete(template_name=template_name))


if __name__ == "__main__":
    list()



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#
"""
Examples:
  dx_delete_vdb.py --vdb aseTest
  dx_delete_vdb.py --vdb aseTest --engine myengine
  --single_thread False --force
"""

from os.path import basename

from delphixpy.v1_10_2 import exceptions
from delphixpy.v1_10_2.web import database
from delphixpy.v1_10_2.web import vo
from dxi._lib import dlpx_exceptions
from dxi._lib import dx_logging
from dxi._lib import get_references
from dxi._lib import run_job
from dxi._lib.run_async import run_async


class _DeleteMixin(object):
    """
    Deletes a VDB or a list of VDBs from an engine
    """

    def _delete_validate_input(self):
        dx_logging.print_debug("Validating inputs for delete option")

        if not self.name:
            dx_logging.print_exception(
                "Please provide the name of an object to delete. "
                "To provide a list of objects, separate them with colons.\n"
                "Eg  name1:name2"
            )
            raise Exception("Please provide the name of an object to delete.")
        if self.db_type.lower() not in ["vdb", "dsource"]:
            raise Exception('Valid dataset types are: "dSource|VDB')

    def _delete_database_helper(self, dlpx_obj):
        """
        :param datasets: List containing datasets to delete
        :type datasets: `list`
        """
        datasets = self.name.split(":")
        for dataset in datasets:
            try:
                container_obj = get_references.find_obj_by_name(
                    dlpx_obj.server_session, database, dataset
                )
                # Check to make sure our container object has a reference
                source_obj = get_references.find_source_by_db_name(
                    dlpx_obj.server_session, dataset
                )
            except Exception as err:
                dx_logging.print_exception(
                    f"Unable to find a reference for {dataset} on "
                    f"engine {dlpx_obj.server_session.address}: {err}"
                )
                container_obj = None
            if container_obj.reference:
                if self.db_type != "vdb" or source_obj.virtual is not True:
                    dx_logging.print_exception(
                        f"Error : {dataset} is not a virtual object."
                    )
                    raise dlpx_exceptions.DlpxException(
                        f"Error : {dataset} is not a virtual object. "
                    )
                elif self.db_type == "dsource" and (
                    source_obj.linked is True or source_obj.staging is True
                ):
                    dx_logging.print_warning(
                        f"DELETING {dataset} THAT "
                        f"IS EITHER A SOURCE OR STAGING DATASET."
                    )
                elif self.db_type not in ["vdb", "dsource"]:
                    dx_logging.print_exception(
                        f"Error : {dataset} is not the "
                        f"provided type: {self.db_type}"
                    )
                    raise dlpx_exceptions.DlpxException(
                        f"Error : {dataset} is not the "
                        f"provided type: {self.db_type}"
                    )
                dx_logging.print_debug(
                    f"Deleting {dataset} on engine "
                    f"{dlpx_obj.server_session.address}"
                )
                delete_params = None
                if self.force and str(container_obj.reference).startswith(
                    "MSSQL"
                ):
                    delete_params = vo.DeleteParameters()
                    delete_params.force = True
                try:
                    dx_logging.print_debug(f"Deleting dataset : {dataset}")
                    database.delete(
                        dlpx_obj.server_session,
                        container_obj.reference,
                        delete_params,
                    )
                    self._add_last_job_to_track(dlpx_obj)
                except (
                    dlpx_exceptions.DlpxException,
                    exceptions.RequestError,
                    exceptions.HttpError,
                ) as err:
                    raise dlpx_exceptions.DlpxException(f"{err}")

    @run_async
    def _delete_main_workflow(self, engine, dlpx_obj, single_thread):
        """
        This function is where we create our main workflow.
        Use the @run_async decorator to run asynchronously.
        The @run_async decorator allows us to run
        against multiple Delphix Engines simultaneously
        :param engine: Dictionary of engines
        :type engine: dictionary
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        :param single_thread:
            True - run single threaded,
            False - run multi-thread
        :type single_thread: bool
        """
        self._delete_validate_input()
        dlpx_obj = self._initialize_session()
        self._setup_dlpx_session(dlpx_obj, engine)
        try:
            with dlpx_obj.job_mode(single_thread):
                self._delete_database_helper(dlpx_obj)
                run_job.track_running_jobs(
                    engine, dlpx_obj, poll=self.poll, failures=self.failures
                )
        except (
            dlpx_exceptions.DlpxException,
            dlpx_exceptions.DlpxObjectNotFound,
            exceptions.RequestError,
            exceptions.JobError,
            exceptions.HttpError,
            Exception,
        ) as err:
            dx_logging.print_exception(
                f"Error in {basename(__file__)} on Delpihx Engine: "
                f'{engine["hostname"]} : {repr(err)}'
            )
            self.failures[0] = True
            # TODO: Add Exception to dictionary



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

from delphixpy.v1_10_2 import exceptions
from delphixpy.v1_10_2.web import environment
from delphixpy.v1_10_2.web import host
from delphixpy.v1_10_2.web import vo
from dxi._lib import dlpx_exceptions
from dxi._lib import dx_logging
from dxi._lib import get_references
from dxi._lib import run_job
from dxi._lib.dxi_constants import EnvironmentOps
from dxi._lib.dxi_constants import EnvironmentTypes
from dxi._lib.dxi_constants import HostTypes
from dxi._lib.run_async import run_async
from dxi._lib.util import DXIConfigConstantsHelper
from dxi.dxi_tool_base import DXIBase
from tabulate import tabulate


class EnvironmentConstants(object):
    """
    Define constants for Environment operations
    """

    SINGLE_THREAD = False
    POLL = 20
    CONFIG = DXIConfigConstantsHelper().get_config()
    LOG_FILE_PATH = DXIConfigConstantsHelper().get_logdir()
    ENGINE_ID = "default"
    PARALLEL = 5
    TYPE = None
    ACTION = None
    HOSTIP = None
    MODULE_NAME = __name__
    LIST_HEADER = [
        "Env Name",
        "Env Type",
        "Env IP",
        "Enabled",
        "Env User",
        "ASE DB User",
    ]


class DXIEnvironment(DXIBase):
    """
    Perform an environment operation
    """

    def __init__(
        self,
        engine=None,
        log_file_path=EnvironmentConstants.LOG_FILE_PATH,
        config_file=EnvironmentConstants.CONFIG,
        poll=EnvironmentConstants.POLL,
        single_thread=EnvironmentConstants.SINGLE_THREAD,
        parallel=EnvironmentConstants.PARALLEL,
        action=EnvironmentConstants.ACTION,
        module_name=EnvironmentConstants.MODULE_NAME,
    ):
        """
        :param engine: Alt Identifier of Delphix engine in dxtools.conf.
        :type engine: `str`
        :param config: The path to the dxtools.conf file
        :type config: `str`
        :param log_file_path: The path to the logfile you want to use.
        :type log_file_path: `str`
        :param poll: The number of seconds to wait between job polls
        :type poll: `int`
        :param single_thread: Run as a single thread.
                              False if running multiple threads.
        :type single_thread: `bool`
        :param action: Operation to perform on the environment.
                       add | update | delete | enable | disable | refresh
        :type action: `str`
        :param env_name: Name of the environment in Delphix
        :type env_name: `str`
        :param env_type: OS type for the environment being added
                         linux | windows
        :type env_type: `str`
        :param toolkit_dir: Toolkit directory on the envrionment
        :type toolkit_dir: `str`
        :param host_ip: IP Address of the host to add
        :type host_ip: `str`
        :param username: Environment OS username
        :type username: `str`
        :param password: Environment user password
        :type password: `str`
        :param connector_env_name: Name of environment with Windows connector
                                   (If adding a Windows environment)
        :type connector_env_name: `str`
        :param old_host: Old IP Address of the environment to update
        :type old_host: `str`
        :param new_host: New IP Address of the environment to update
        :type new_host_: `str`
        :param ase_db_username: ASE DB user
        :type ase_db_username: `str`
        :param ase_db_username: ASE DB user's password
        :type ase_db_username: `str`
        """
        super().__init__(
            parallel=parallel,
            poll=poll,
            config=config_file,
            log_file_path=log_file_path,
            single_thread=single_thread,
            engine=engine,
            module_name=module_name,
        )
        self.action = action
        # self._validate_input()

    def _validate_input(self):
        if not self.action:
            dx_logging.print_exception(
                f"Invalid action provided."
                f"Select at least one option "
                f"(list | add | updatehost | "
                f"delete | refresh | enable | disable)"
            )
            raise

        if self.action == "add":
            if (
                self.env_name is None
                or self.env_type is None
                or self.host_ip is None
                or self.toolkit_dir is None
                or self.username is None
                or self.password is None
            ):
                dx_logging.print_exception(
                    f"Missing input for environment add operation"
                    f"Required parameters: "
                    f"(envname , envtype , hostip , "
                    f"toolkitdir, username, password )"
                )
                raise
            elif (
                self.env_type == "windows" and self.connector_env_name is None
            ):
                dx_logging.print_exception(
                    f"connector_env_name is reqquired "
                    f"to add a Windows environment"
                )
                raise

        if self.action == "updatehost" and (
            self.old_host is None or self.new_host is None
        ):
            dx_logging.print_exception(
                f"Missing required input for updatehost operation"
                f"Required parameters: "
                f"( oldhost , newhost )"
            )
            raise

    def _environment_execution_helper(self):
        """
        Execution helper for all environment operations
        """
        self.display_choices(self)
        for each in run_job.run_job_mt(
            self.main_workflow,
            self.dx_session_obj,
            self.engine,
            self.single_thread,
        ):
            each.join()
        elapsed_minutes = run_job.time_elapsed(self.time_start)
        dx_logging.print_info(
            f"Environment operation took {elapsed_minutes}"
            f" minutes to complete."
        )

    def list(self):
        """
        List all existing environments
        """
        self.action = EnvironmentOps.LIST
        try:
            self._environment_execution_helper()
            dx_logging.print_debug("End of Execution")
            return True
        except Exception as err:
            dx_logging.print_exception(
                f"An Error was encountered during "
                f"environment list operation: {repr(err)}"
            )
            return False

    def add(
        self,
        env_name=None,
        env_type=EnvironmentConstants.TYPE,
        host_ip=EnvironmentConstants.HOSTIP,
        toolkit_dir=None,
        os_user=None,
        os_user_pwd=None,
        connector_env_name=None,
        ase_db_user=None,
        ase_db_user_pwd=None,
    ):
        """
        :param env_name: Name of the environment in Delphix
        :type env_name: `str`
        :param env_type: OS type for the environment being added
                         linux | windows
        :type env_type: `str`
        :param toolkit_dir: Toolkit directory on the envrionment
        :type toolkit_dir: `str`
        :param host_ip: IP Address of the host to add
        :type host_ip: `str`
        :param username: Environment OS username
        :type username: `str`
        :param password: Environment user password
        :type password: `str`
        :param connector_env_name: Name of environment with Windows connector
                                   (If adding a Windows environment)
        :type connector_env_name: `str`
        """
        self.action = EnvironmentOps.ADD
        self.env_name = env_name
        self.env_type = env_type
        self.host_ip = host_ip
        self.toolkit_dir = toolkit_dir
        self.username = os_user
        self.password = os_user_pwd
        self.connector_env_name = connector_env_name
        self.ase_db_username = ase_db_user
        self.ase_db_password = ase_db_user_pwd
        try:
            self._environment_execution_helper()
            dx_logging.print_debug(
                "Adding environment to Delphix Engine complete."
            )
            return True
        except Exception as err:
            dx_logging.print_exception(
                f"An Error was encountered during "
                f"environment add operation: {repr(err)}"
            )
            return False

    def enable(self, env_name=None):
        """
        :param env_name: Name of the environment in Delphix
        :type env_name: `str`
        """
        self.env_name = env_name
        self.action = EnvironmentOps.ENABLE
        try:
            self._environment_execution_helper()
            dx_logging.print_debug("End of Execution")
            return True
        except Exception as err:
            dx_logging.print_exception(
                f"An Error was encountered during "
                f"environment enable operation: {repr(err)}"
            )
            return False

    def disable(self, env_name=None):
        """
        :param env_name: Name of the environment in Delphix
        :type env_name: `str`
        """
        self.env_name = env_name
        self.action = EnvironmentOps.DISABLE
        try:
            self._environment_execution_helper()
            dx_logging.print_debug("End of Execution")
            return True
        except Exception as err:
            dx_logging.print_exception(
                f"An Error was encountered during "
                f"environment disable operation: {repr(err)}"
            )
            return False

    def delete(self, env_name=None):
        """
        :param env_name: Name of the environmentin Delphix
        :type env_name: `str`
        """
        self.env_name = env_name
        self.action = EnvironmentOps.DELETE
        try:
            self._environment_execution_helper()
            dx_logging.print_debug("End of Execution")
            return True
        except Exception as err:
            dx_logging.print_exception(
                f"An Error was encountered during "
                f"environment delete operation: {repr(err)}"
            )
            return False

    def refresh(self, env_name=None):
        """
        :param env_name: Name of the environment in Delphix
        :type env_name: `str`
        """
        self.env_name = env_name
        self.action = EnvironmentOps.REFRESH
        try:
            self._environment_execution_helper()
            dx_logging.print_debug("End of Execution")
            return True
        except Exception as err:
            dx_logging.print_exception(
                f"An Error was encountered during "
                f"environment refresh operation: {repr(err)}"
            )
            return False

    def updatehost(self, old_host=None, new_host=None):
        """
        :param old_host: Old IP Address of the environment to update
        :type old_host: `str`
        :param new_host: New IP Address of the environment to update
        :type new_host_: `str`
        """
        self.old_host = old_host
        self.new_host = new_host
        self.action = EnvironmentOps.UPDATEHOST
        try:
            self._environment_execution_helper()
            dx_logging.print_debug("End of Execution")
            return True
        except Exception as err:
            dx_logging.print_exception(
                f"An Error was encountered during "
                f"environment updatehost operation: {repr(err)}"
            )
            return False

    def _enable_environment(self, dlpx_obj):
        """
        Enable the given host
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        :param env_name: Environment name in Delphix
        :type env_name: str
        """
        env_obj = get_references.find_obj_by_name(
            dlpx_obj.server_session, environment, self.env_name
        )
        try:
            environment.enable(dlpx_obj.server_session, env_obj.reference)
            self._add_last_job_to_track(dlpx_obj)
        except (dlpx_exceptions.DlpxException, exceptions.RequestError) as err:
            dx_logging.print_exception(
                f"ERROR: Enabling the host {self.env_name} "
                f"encountered an error: {err}"
            )
            raise err

    def _disable_environment(self, dlpx_obj):
        """
        Disable a Delphix environment
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        :param env_name: Environment name in Delphix
        :type env_name: str
        """
        env_obj = get_references.find_obj_by_name(
            dlpx_obj.server_session, environment, self.env_name
        )
        try:
            environment.disable(dlpx_obj.server_session, env_obj.reference)
            self._add_last_job_to_track(dlpx_obj)
        except (dlpx_exceptions.DlpxException, exceptions.RequestError) as err:
            dx_logging.print_exception(
                f"ERROR: Disabling the host {self.env_name}"
                f"encountered an error:{err}"
            )
            raise err

    def _update_host_environment(self, dlpx_obj):
        """
        Update the environment
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        :param old_host_address: Original IP address of environment
        :type old_host_address: str
        :param new_host_address: New IP address of the environment
        :type new_host_address: str
        """
        old_host_obj = get_references.find_obj_by_name(
            dlpx_obj.server_session, host, self.old_host
        )
        if old_host_obj.type == HostTypes.WIN.value:
            host_obj = vo.WindowsHost()
        else:
            host_obj = vo.UnixHost()
        host_obj.address = self.new_host
        try:
            host.update(
                dlpx_obj.server_session, old_host_obj.reference, host_obj
            )
            self._add_last_job_to_track(dlpx_obj)
        except (dlpx_exceptions.DlpxException, exceptions.RequestError) as err:
            dx_logging.print_exception(
                f"ERROR: Updating the host {host_obj.name} "
                f"encountered an error:{err}"
            )

    def _list_env(self, dlpx_obj):
        """
        List all environments for the engine
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        """
        all_envs = environment.get_all(dlpx_obj.server_session)
        if not all_envs:
            dx_logging.print_info(
                f"There are no environments on "
                f"engine:{dlpx_obj.server_session.address}"
            )
            return
        env_host = ""
        ase_params = ""
        print_list = []
        env_list = []
        for env in all_envs:
            env_user = get_references.find_obj_name(
                dlpx_obj.server_session, environment.user, env.primary_user
            )
            try:
                env_host = get_references.find_obj_name(
                    dlpx_obj.server_session, host, env.host
                )
            except AttributeError:
                pass
            try:
                if hasattr(env, "env.ase_host_environment_parameters"):
                    if isinstance(
                        env.env.ase_host_environment_parameters,
                        vo.ASEHostEnvironmentParameters,
                    ):
                        ase_params = (
                            env.env.ase_host_environment_parameters.db_user
                        )
            except Exception:
                dx_logging.print_exception(
                    "Exception retrieving ASE env info."
                )
            print_list.append(
                [
                    env.name,
                    env.type,
                    env_host,
                    env.enabled,
                    env_user,
                    ase_params,
                ]
            )
            env_list.append(
                dict(
                    zip(
                        EnvironmentConstants.LIST_HEADER,
                        [
                            env.name,
                            env_user,
                            env_host,
                            env.enabled,
                            env_user,
                            ase_params,
                        ],
                    )
                )
            )
        print(
            tabulate(
                print_list,
                headers=EnvironmentConstants.LIST_HEADER,
                tablefmt="grid",
            )
        )
        return env_list

    def _delete_env(self, dlpx_obj):
        """
        Deletes an environment
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        :param env_name: Name of the environment to delete
        :type env_name: str
        """
        env_obj = get_references.find_obj_by_name(
            dlpx_obj.server_session, environment, self.env_name
        )
        if env_obj:
            environment.delete(dlpx_obj.server_session, env_obj.reference)
            self._add_last_job_to_track(dlpx_obj)
        elif env_obj is None:
            dlpx_exceptions.DlpxObjectNotFound(
                f"Environment was not found: {self.env_name}"
            )

    def _refresh_env(self, dlpx_obj):
        """
        Refresh the environment
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        :parm env_name: Name of the environment to refresh
        :type env_name: str
        """
        if self.env_name == "all":
            env_list = get_references.find_all_objects(
                dlpx_obj.server_session, environment
            )
            for env_obj in env_list:
                try:
                    environment.refresh(
                        dlpx_obj.server_session, env_obj.reference
                    )
                    if dlpx_obj.server_session.last_job:
                        dlpx_obj.jobs[dlpx_obj.server_session.address].append(
                            dlpx_obj.server_session.last_job
                        )
                except (
                    dlpx_exceptions.DlpxException,
                    exceptions.RequestError,
                ) as err:
                    dlpx_exceptions.DlpxException(
                        f"Encountered an error while "
                        f"refreshing {self.env_name}: {err}"
                    )
        else:
            try:
                env_obj = get_references.find_obj_by_name(
                    dlpx_obj.server_session, environment, self.env_name
                )
                environment.refresh(dlpx_obj.server_session, env_obj.reference)
                self._add_last_job_to_track(dlpx_obj)
            except (
                dlpx_exceptions.DlpxException,
                exceptions.RequestError,
            ) as err:
                dx_logging.print_exception(
                    f"Refreshing {self.env_name} encountered an error:{err}"
                )
                raise dlpx_exceptions.DlpxException(
                    f"Refreshing {self.env_name} encountered an error:\n{err}"
                )

    def _add_linux_env(self, dlpx_obj):
        """
        Add a Linux environment.
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        """
        env_params_obj = vo.HostEnvironmentCreateParameters()
        env_params_obj.host_environment = vo.UnixHostEnvironment()
        env_params_obj.host_parameters = vo.UnixHostCreateParameters()
        env_params_obj.host_parameters.host = vo.UnixHost()
        env_params_obj.host_environment.name = self.env_name
        env_params_obj.host_parameters.host.address = self.host_ip
        env_params_obj.host_parameters.name = self.env_name
        env_params_obj.host_parameters.host.toolkit_path = self.toolkit_dir
        # setting user credentials
        env_params_obj.primary_user = vo.EnvironmentUser()
        env_params_obj.primary_user.name = self.username

        if self.password is None:
            env_params_obj.primary_user.credential = vo.SystemKeyCredential()
        else:
            env_params_obj.primary_user.credential = vo.PasswordCredential()
            env_params_obj.primary_user.credential.password = self.password
        if self.ase_db_username:
            env_params_obj.host_environment.ase_host_environment_parameters = (  # noqa
                vo.ASEHostEnvironmentParameters()
            )
            env_params_obj.host_environment.ase_host_environment_parameters.db_user = (  # noqa
                self.ase_db_username
            )
            env_params_obj.host_environment.ase_host_environment_parameters.credentials = (  # noqa
                vo.PasswordCredential()
            )
            env_params_obj.host_environment.ase_host_environment_parameters.credentials.password = (  # noqa
                self.ase_db_password
            )
        try:
            environment.create(dlpx_obj.server_session, env_params_obj)
            self._add_last_job_to_track(dlpx_obj)
        except (
            dlpx_exceptions.DlpxException,
            exceptions.RequestError,
            exceptions.HttpError,
        ) as err:
            raise dlpx_exceptions.DlpxException(
                f"ERROR: Encountered an exception while adding the "
                f"environment:{err}"
            )
        except exceptions.JobError as err:
            raise dlpx_exceptions.DlpxException(
                f"JobError while creating environment:{err}"
            ) from err

    def _add_windows_env(self, dlpx_obj):
        """
        Create a Windows environment.
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        """
        env_params_obj = vo.HostEnvironmentCreateParameters()
        env_params_obj.primary_user = vo.EnvironmentUser()
        env_params_obj.primary_user.name = self.username
        env_params_obj.primary_user.credential = vo.PasswordCredential()
        env_params_obj.primary_user.credential.password = self.password
        env_params_obj.host_parameters = vo.WindowsHostCreateParameters()
        env_params_obj.host_parameters.host = vo.WindowsHost()
        env_params_obj.host_parameters.host.address = self.host_ip
        env_params_obj.host_parameters.host.connector_port = 9100
        env_params_obj.host_environment = vo.WindowsHostEnvironment()
        env_params_obj.host_environment.name = self.env_name
        env_obj = None
        if self.connector_env_name:
            env_obj = get_references.find_obj_by_name(
                dlpx_obj.server_session, environment, self.connector_env_name
            )
        if env_obj:
            env_params_obj.host_environment.proxy = env_obj.host
        elif self.connector_env_name is not None and env_obj is None:
            raise dlpx_exceptions.DlpxObjectNotFound(
                f"Environment:{self.connector_env_name} "
                f"was not found on the Engine"
            )
        try:
            environment.create(dlpx_obj.server_session, env_params_obj)
            self._add_last_job_to_track(dlpx_obj)
        except (
            dlpx_exceptions.DlpxException,
            exceptions.RequestError,
            exceptions.HttpError,
        ) as err:
            raise dlpx_exceptions.DlpxException(
                f"ERROR: Encountered an exception while adding the "
                f"environment: {err}"
            )

    @run_async
    def main_workflow(self, engine, dlpx_obj, single_thread):
        """
        This function is where we create our main workflow.
        Use the @run_async decorator to run this function asynchronously.
        The @run_async to run against multiple Delphix Engine
        simultaneously
        :param engine: Dictionary of engines
        :type engine: dictionary
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        :param single_thread: True - run single threaded,
                              False - run multi-thread
        :type single_thread: bool
        """
        dlpx_obj = self._initialize_session()
        try:
            dlpx_obj.dlpx_session(
                engine["ip_address"],
                engine["username"],
                engine["password"],
                engine["use_https"],
            )
        except dlpx_exceptions.DlpxException as err:
            dx_logging.print_exception(
                f"ERROR: dx_environment encountered an error "
                f'authenticating to {engine["hostname"]}: {err}'
            )
        try:
            with dlpx_obj.job_mode(single_thread):
                if self.action == EnvironmentOps.LIST:
                    self._list_env(dlpx_obj)
                elif self.action == EnvironmentOps.ADD:
                    if self.env_type.lower() == EnvironmentTypes.WIN.value:
                        self._add_windows_env(dlpx_obj)
                    else:
                        self._add_linux_env(dlpx_obj)
                elif self.action == EnvironmentOps.ENABLE:
                    self._enable_environment(dlpx_obj)
                elif self.action == EnvironmentOps.DISABLE:
                    self._disable_environment(dlpx_obj)
                elif self.action == EnvironmentOps.DELETE:
                    self._delete_env(dlpx_obj)
                elif self.action == EnvironmentOps.REFRESH:
                    self._refresh_env(dlpx_obj)
                elif self.action == EnvironmentOps.UPDATEHOST:
                    self._update_host_environment(dlpx_obj)
                run_job.track_running_jobs(
                    engine, dlpx_obj, poll=self.poll, failures=self.failures
                )
        except (
            dlpx_exceptions.DlpxException,
            exceptions.RequestError,
            exceptions.JobError,
            exceptions.HttpError,
            Exception,
        ) as err:
            dx_logging.print_exception(
                f"Error in dx_environment on engine:"
                f'{engine["hostname"]}: Error Message: {err}'
            )
            self.failures[0] = True



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#
"""
This module includes classes for
Delphix Self Service Template Operations

Examples:
  dxi_template.py --list_templates
  dxi_template.py --create_template jstemplate1 --database <name>
  dxi_template.py --create_template jstemplate2 --database <name:name:name>
  dxi_template.py --delete_template jstemplate1

"""

from delphixpy.v1_10_2 import exceptions
from delphixpy.v1_10_2.web import database
from delphixpy.v1_10_2.web import selfservice
from delphixpy.v1_10_2.web import vo
from dxi._lib import dlpx_exceptions as dxe
from dxi._lib import dx_logging as log
from dxi._lib import get_references as ref
from dxi._lib import run_job
from dxi._lib.dxi_constants import TemplateOps
from dxi._lib.run_async import run_async
from dxi._lib.util import DXIConfigConstantsHelper
from dxi.dxi_tool_base import DXIBase
from tabulate import tabulate


class DXITemplateConstants(object):
    """
    Define constants for Self Service Template Class & CLI
    """

    SINGLE_THREAD = False
    POLL = 20
    CONFIG = DXIConfigConstantsHelper().get_config()
    LOG_FILE_PATH = DXIConfigConstantsHelper().get_logdir()
    ENGINE_ID = "default"
    PARALLEL = 5
    ACTION = None
    MODULE_NAME = __name__
    LIST_HEADER = ["Name", "Reference", "Active Branch", "Last Updated"]


class DXITemplate(DXIBase):
    """
    Class for Self Service Template Operations
    """

    def __init__(
        self,
        engine=DXITemplateConstants.ENGINE_ID,
        single_thread=DXITemplateConstants.SINGLE_THREAD,
        config=DXITemplateConstants.CONFIG,
        log_file_path=DXITemplateConstants.LOG_FILE_PATH,
        poll=DXITemplateConstants.POLL,
        action=DXITemplateConstants.ACTION,
    ):
        """
        :param engine: Identifier for the engine in dxtools.conf
        :type engine: `str`

        :param single_thread: Run jobs synchronously on an engine
                              True forSync Mode| False for Async Mode
        :type single_thread: `bool`

        :param config: The path to the dxtools.conf file
        :type config: `str`

        :param log_file_path: The path to the logfile for this module
        :type log_file_path: `str`

        :param poll: Time in seconds to poll status of a job.
        :type log_file_path: `int
        """
        super().__init__(
            config=config,
            log_file_path=log_file_path,
            single_thread=single_thread,
            engine=engine,
            poll=poll,
            module_name=__name__,
        )
        self.action = action

    def __validate_input(self):
        pass

    def list(self):
        """
        List all templates on an engine
        """
        self.action = TemplateOps.CREATE
        try:
            self._execute_operation(self._list_template_helper)
            return True
        except Exception:
            return False

    def create(self, template_name=None, dbnames=None):
        """
        Create a template on an engine

        :param template_name: Name of the template to create
        :type template_name: str

        :param dbnames: List of datasource names, separated by ":'
                        Sample oraclesrc1:sqlsrc1
        :type dbnames: str

        """
        self.action = TemplateOps.CREATE
        self.template_name = template_name
        self.dbnames = dbnames
        try:
            self._execute_operation(self._create_template_helper)
            return True
        except Exception:
            return False

    def delete(self, template_name=None):
        """
        Delete a template from an engine

        :param template_name: Name of the template to delete
        :type template_name: str
        """
        self.action = TemplateOps.CREATE
        self.template_name = template_name
        try:
            self._execute_operation(self._delete_template_helper)
            return True
        except Exception:
            return False

    @run_async
    def _list_template_helper(self, engine_ref, dlpx_obj, single_thread):
        """
        Execution helper for template list operation

        :param engine_ref: Dictionary of engines
        :type engine_ref: `dict`

        :param dlpx_obj: DDP session object
        :type dlpx_obj: `lib.GetSession.GetSession`

        :return: List of all templtes on a given engine
        :rtype: `list`
        """
        dlpx_obj = self._initialize_session()
        self._setup_dlpx_session(dlpx_obj, engine_ref)
        try:
            print_list = []
            with dlpx_obj.job_mode(self.single_thread):
                container_list = []
                ss_templates = selfservice.template.get_all(
                    dlpx_obj.server_session
                )
                if not ss_templates:
                    log.print_info(f"No Self Service Template on engine")
                else:
                    for ss_template in ss_templates:
                        last_updated = ref.convert_timestamp(
                            dlpx_obj.server_session,
                            ss_template.last_updated[:-5],
                        )
                        print_list.append(
                            [
                                ss_template.name,
                                ss_template.reference,
                                ss_template.active_branch,
                                last_updated,
                            ]
                        )
                        container_list.append(
                            dict(
                                zip(
                                    DXITemplateConstants.LIST_HEADER,
                                    [
                                        ss_template.name,
                                        ss_template.reference,
                                        ss_template.active_branch,
                                        last_updated,
                                    ],
                                )
                            )
                        )
                    self._add_last_job_to_track(dlpx_obj)
                    run_job.track_running_jobs(
                        engine_ref,
                        dlpx_obj,
                        poll=self.poll,
                        failures=self.failures,
                    )
                    print(
                        tabulate(
                            print_list,
                            headers=DXITemplateConstants.LIST_HEADER,
                            tablefmt="grid",
                        )
                    )
            return container_list
        except (
            dxe.DlpxException,
            exceptions.HttpError,
            exceptions.RequestError,
        ) as err:
            log.print_exception(
                f"ERROR: The templates could not be listed: {err}"
            )
            # raise dxe.DlpxException(
            #     f"ERROR: The templates could not be listed: {err}"
            # )
            self.failures[0] = True

    @run_async
    def _create_template_helper(self, engine_ref, dlpx_obj, single_thread):
        """
        Execution helper for template create operation

        :param engine_ref: Dictionary of engines
        :type engine_ref: `dict`

        :param dlpx_obj: DDP session object
        :type dlpx_obj: `lib.GetSession.GetSession`

        :return: created template reference
        :rtype: `str`
        """
        dlpx_obj = self._initialize_session()
        self._setup_dlpx_session(dlpx_obj, engine_ref)
        with dlpx_obj.job_mode(self.single_thread):
            try:
                ss_template_params = vo.JSDataTemplateCreateParameters()
                ss_template_params.name = self.template_name
                template_ds_lst = []
                template_ref = ""
                for dbname in self.dbnames.split(":"):
                    template_ds_lst.append(
                        ref.build_data_source_params(
                            dlpx_obj, database, dbname
                        )
                    )
                ss_template_params.data_sources = template_ds_lst
                template_ref = selfservice.template.create(
                    dlpx_obj.server_session, ss_template_params
                )
                self._add_last_job_to_track(dlpx_obj)
                run_job.track_running_jobs(
                    engine_ref,
                    dlpx_obj,
                    poll=self.poll,
                    failures=self.failures,
                )
                return template_ref
            except (
                dxe.DlpxException,
                exceptions.RequestError,
                exceptions.HttpError,
            ) as err:
                log.print_exception(
                    f"ERROR: The template "
                    f"{self.template_name} was not created: {err}"
                )
                # raise dxe.DlpxException(
                #     f"ERROR: The template "
                #     f"{self.template_name} was not created: {err}"
                # )
                self.failures[0] = True

    @run_async
    def _delete_template_helper(self, engine_ref, dlpx_obj, single_thread):
        """
        Execution helper for template delete operation

        :param engine_ref: Dictionary of engines
        :type engine_ref: `dict`

        :param dlpx_obj: DDP session object
        :type dlpx_obj: `lib.GetSession.GetSession`
        """
        dlpx_obj = self._initialize_session()
        self._setup_dlpx_session(dlpx_obj, engine_ref)
        with dlpx_obj.job_mode(self.single_thread):
            try:
                template_ref = ref.find_obj_by_name(
                    dlpx_obj.server_session,
                    selfservice.template,
                    self.template_name,
                ).reference
                selfservice.template.delete(
                    dlpx_obj.server_session, template_ref
                )
                self._add_last_job_to_track(dlpx_obj)
                run_job.track_running_jobs(
                    engine_ref,
                    dlpx_obj,
                    poll=self.poll,
                    failures=self.failures,
                )
            except (Exception, BaseException) as err:
                log.print_exception(
                    f"Template {self.template_name} was not deleted. "
                    f"ERROR:{str(err)}"
                )
                # raise dxe.DlpxException(
                #     f"ERROR: The template "
                #     f"{self.template_name} was not deleted: {err}"
                # )
                self.failures[0] = True



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

import click
from dxi._cli._bookmark_cli import bookmark
from dxi._cli._branch_cli import branch
from dxi._cli._config_cli import config
from dxi._cli._container_cli import container
from dxi._cli._database_cli import database
from dxi._cli._environment_cli import environment
from dxi._cli._snapshot_cli import snapshot
from dxi._cli._template_cli import template
from dxi._version import __version__

@click.group()
@click.version_option(version=__version__, prog_name="dxi")
def dxi():
    """
    dxi is a command line interface to trigger Delphix operations and to
    integrate Delphix into automated workflows.
    """


# Adding snapshot as sub-group-command of dxi
dxi.add_command(snapshot)

# Adding snapshot as sub-group-command of dxi
dxi.add_command(database)

# Adding environment as sub-group-command of dxi
dxi.add_command(environment)

# Adding branch as sub-group-command of dxi
dxi.add_command(branch)

# Adding bookmark as sub-group-command of dxi
dxi.add_command(bookmark)

# Adding container as sub-group-command of dxi
dxi.add_command(container)

# Adding template as sub-group-command of dxi
dxi.add_command(template)

# Adding config as sub-group-command of dxi
dxi.add_command(config)

if __name__ == "__main__":
    dxi()



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

import json
import os
import sys
import uuid

from cryptography.exceptions import InvalidTag
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.ciphers.aead import ChaCha20Poly1305
from delphixpy import exceptions
from dxi._lib import dlpx_exceptions
from dateutil import parser

from . import dx_logging as log

class DXIConfigConstantsHelper(object):
    DXIDATA = "dxi-data"
    CONF = '/config/dxtools.conf'
    LOG= '/logs/'

    ROOT_DEFAULT = '../../'

    def get_dxidata_root(self):
        root = os.environ.get('DXI_DIR', self.ROOT_DEFAULT)

        if not root.endswith('/'):
            root = root + '/'

        if root == '/dxi-data/':
            return root
        else:
            return root + self.DXIDATA

    def get_config(self):
        return self.get_dxidata_root() + self.CONF

    def get_logdir(self):
        return self.get_dxidata_root() + self.LOG

def find_config_path(config_file_path):
    """
    Locate the config path if the default is passed.

    :param config_file_path: Path to the configuration file.
    :type config_file_path: `str`
    :return: config_file_path
    :rtype: `str`
    """
    try:
        if os.path.isfile(config_file_path) and os.path.isabs(
                config_file_path
        ):
            return config_file_path
        else:
            exec_path = get_executable_path()
            if exec_path is not None:
                config_file_path = str(
                    os.path.join(exec_path, "dxi-data/config/dxtools.conf")
                )
            else:
                log.print_debug(
                    f"Config location passed to dxi does not exist. "
                    f"Scanning default location."
                )
                config_file_path = DXIConfigConstantsHelper().get_config()

            if not os.path.isfile(config_file_path):
                err_msg = (
                    f"Error: Could not find dxtools.conf at the "
                    f"default location or the passed location,\n "
                    f"dxi requires that you provide a full path "
                    f"to dxtools.conf file (including filename)"
                )
                log.print_exception(err_msg)
                raise Exception(err_msg)
        log.print_debug("Config file path is: {}".format(config_file_path))
        return config_file_path
    except Exception as err:
        raise Exception(
            f"Failed to find config path @ {config_file_path} {repr(err)}"
        )


def find_log_path(log_path, module_name):
    """
    Locate the log path if the default is passed.
    This logic will be changed once log_path is property of parent class.

    :param config_file_path: Path to the configuration file.
    :type config_file_path: `str`
    :param module_name: Module name
    :type module_name: `str`
    :return: Log file path
    :rtype: `str`
    """
    log_file_path = None
    try:
        if os.path.isfile(log_path):
            return log_path
        else:
            exec_path = get_executable_path()
            if exec_path is not None:
                # do something
                log_file_path = str(os.path.join(exec_path, "dxi-data/logs"))
            else:
                log.print_debug(
                    f"Logs location passed to dxi does not exist. "
                    f"Using default location."
                )
                log_file_path = DXIConfigConstantsHelper().get_logdir()
                
            if not os.path.isdir(log_file_path):
                err_msg = (
                    f"Error: Could not find logs directory "
                    f"at default location or the passed location.\n "
                    f"Please review the inputs and retry)"
                )
                log.print_exception(err_msg)
                raise Exception(err_msg)
            log_file_path = os.path.abspath(log_file_path)
            log_file_path = str(
                os.path.join(log_file_path, "{}.log".format(module_name))
            )
            log.print_debug("Log file path is: {}".format(log_file_path))

        return log_file_path
    except Exception as err:
        raise Exception(
            f"Failed to find log path @ {log_file_path} {repr(err)}"
        )



def run_async(func):
    """
    http://code.activestate.com/recipes/576684-simple-threading-decorator/
    run_async(func)
    function decorator, intended to make "func" run in a separate
    thread (asynchronously).
    Returns the created Thread object

    E.g.:
    @run_async
    def task1():
        do_something

    @run_async
    def task2():
        do_something_too

        t1 = task1()
        t2 = task2()
        ...
        t1.join()
        t2.join()

    :param func: The callable function object to be invoked by the run
    :type func: `callable`
    :return: Decorated function
    :rtype: `callable`

    """
    from threading import Thread
    from functools import wraps

    @wraps(func)
    def async_func(*args, **kwargs):
        func_hl = Thread(target=func, args=args, kwargs=kwargs)
        func_hl.start()
        return func_hl

    return async_func


def exception_handler(function):
    def inner_function(*args, **kwargs):
        try:
            function(*args, **kwargs)
            # Here we handle what we do when the unexpected happens
            return True
        except SystemExit as err:
            # This is what we use to handle our sys.exit(#)
            log.print_exception(
                f"ERROR: Please check the ERROR message below:\n{err}"
            )
        except dlpx_exceptions.DlpxException as err:
            # We use this exception handler when an error occurs in a function.
            log.print_exception(
                f"ERROR: Please check the ERROR message below:\n{err}"
            )
        except exceptions.HttpError as err:
            # We use this exception handler when our connection to Delphix
            # fails
            print(
                f"ERROR: Connection failed to the Delphix Engine. Please "
                f"check the error message below:\n{err}"
            )
        except exceptions.JobError as err:
            # We use this exception handler when a job fails in Delphix so
            # that we have actionable data
            print(f"A job failed in the Delphix Engine:\n{err.job}")
        except KeyboardInterrupt:
            # We use this exception handler to gracefully handle ctrl+c exits
            log.print_debug(
                "You sent a CTRL+C to interrupt the process"
            )
        return False

    return inner_function


def convert_dct_str(obj_dct):
    """
    Convert dictionary into a string for printing
    :param obj_dct: Dictionary to convert into a string
    :type obj_dct: dict
    :return: string object
    """
    js_str = ""
    if isinstance(obj_dct, dict):
        for js_db, js_jdbc in obj_dct.items():
            if isinstance(js_jdbc, list):
                js_str += f'{js_db}: {", ".join(js_jdbc)}\n'
            elif isinstance(js_jdbc, str):
                js_str += f"{js_db}: {js_jdbc}\n"
    else:
        raise dlpx_exceptions.DlpxException(
            f"Passed a non-dictionary object to convert_dct_str():"
            f"{type(obj_dct)}"
        )
    return js_str


def boolean_based_system_exit(bool_status):
    """
    Exit with correct code based on boolean value
    :param bool_status:
    :type bool_status
    """
    if bool_status:
        sys.exit(0)
    else:
        sys.exit(1)


def show_progress(per, jobid, with_progress_bar=True, status='Running', total=100):
    if with_progress_bar:
        prefix = 'Progress(' + jobid + ')'
        if per >= total:
            status = 'Completed'
        bar_len = 60
        filled_len = int(round(per * bar_len / float(total)))
        bar = '=' * filled_len + '-' * (bar_len - filled_len)
        fmt = f'\r{prefix} |{bar}| {per}% {status}'
        print('\b' * len(fmt), end='')
        sys.stdout.write(fmt)
        sys.stdout.flush()


def decrypt_data(key, value):
    """
    Decryts a value with SHA265 agorithm
    :param value: value to decrypt
    return decrypted value
    """
    hashhex = value[-64:]
    enc = value[0:-64]
    try:
        dehex = bytes.fromhex(enc)
        msg_nonce = dehex[:12]
        ciphertext = dehex[12:]
        cipher = ChaCha20Poly1305(key)
        value = cipher.decrypt(msg_nonce, ciphertext, None)
        hash_object = hashes.Hash(hashes.SHA256(), backend=default_backend())
        hash_object.update(value)
        dhash = hash_object.finalize()
        hashhex_check = dhash.hex()
        if hashhex == hashhex_check:
            return value.decode('utf-8')
        else:
            raise Exception("Password SHA256 value after decrypt is wrong")
    except InvalidTag as err:
        log.print_exception(f"Wrong decryption key: {err}")
        raise err
    except Exception as err:
        log.print_exception(f"Error: Incorrect Decrypted Value: {err}")
        raise err

def encrypt_data(key,password):
    """
    Encrypts a value with SHA 256 algorithm
    :param value : Value to encrypt
    return encryted value
    """
    try:
        hash_object = hashes.Hash(hashes.SHA256(), backend=default_backend())
        hash_object.update(password.encode())
        dhash = hash_object.finalize()
        hashhex = dhash.hex()
        nonce = os.urandom(12)
        cipher = ChaCha20Poly1305(key)
        enc = cipher.encrypt(nonce, password.encode(), None)
        enchex = enc.hex()
        return nonce.hex() + enchex + hashhex
    except Exception as err:
        log.print_exception(f"Error: There as an exception while encrypting {err}")
        raise err

def get_encryption_key(seed):
    try:
        key = hex(uuid.getnode())+seed
        key = '{:32.32}'.format(key)
        key = key.encode()
        return key
    except Exception as err:
        log.print_exception(f"Error: There was an error creating key: {err}")
        raise


def init_global_config_folders(config, log_dir):
    """
    Called as part of init operation.
    Does the following
        - Creates the config directory if it does not exist
        - Creates Log Directory if it does not exist.
    Args:
         config_dir: Full path to the config directory
         log_dir: Full path to the logs directory

    Raises:
        General Exception
    """
    try:
        exec_path = get_executable_path()
        if exec_path is not None:
            config_path = str(
                os.path.join(exec_path, "dxi-data/config/dxtools.conf")
            )
            log_path = str(os.path.join(exec_path, "dxi-data/logs"))
        else:
            log_path = get_abs_path(log_dir)
            config_path = get_abs_path(config)
    except Exception as err:
        log.print_exception(
            f" There was an exception while "
            f"resolving config and log root directories {err}"
        )
        raise Exception(
            f"There was an exception while "
            f"resolving config and log root directories {err}"
        )
    try:
        log.print_info("Log Directory full path " + log_path)
        if not os.path.exists(log_path):
            os.makedirs(log_path, exist_ok=True)
        log.print_info(f"Log directory created @ {log_path}")
    except Exception as err:
        log.print_exception(
            f"init_global_config_folders(): "
            f"There was an error while creating log directory: {err}"
        )
        raise err
    try:
        log.print_info("Config Directory full path " + config_path)
        if not os.path.exists(config_path):
            os.makedirs(os.path.dirname(config_path), exist_ok=True)
        # copy2(get_abs_path("../config/dxtools.conf"), config_path)
        _add_default_config_value(config_path)
        log.print_debug(f"Config folder created at @ {config_path}")
        # log.warn(
        #     f"ACTION REQUIRED:"
        #     f"\n\tPlease create a dxtools.conf file @ {config_path}"
        #     f"\n\tand add your Delphix Engine information into the file."
        # )
    except Exception as err:
        log.print_exception(
            f"init_global_config_folders(): "
            f"There was an exception while creating config file : {err}"
        )
        raise err


def get_abs_path(relative_path, base_dir=__file__):
    """
    Returns the absolute path
    Args:
        base_dir: Base directory for relative path
        relative_path: Relative path
    Returns:
        abs_path: Absolute path for relative path
    Raises:
        Exception: General Exception
    """
    if str(relative_path).startswith("/"):
        return relative_path

    try:
        path = os.path.dirname(base_dir)
        levels = relative_path.split("/")
        for level in levels:
            path = os.path.join(path, level)
        abs_path = os.path.abspath(path)
        return str(abs_path)
    except Exception as err:
        log.print_exception(
            f"There was an error getting "
            f"the absolute path for: {relative_path}: {err}"
        )

def get_executable_path():
    """
    Check if this is an executable distribution.
    If executable, return the path of the exe file.

    Returns:
        path: Executable location
    """
    exec_dir = None
    if getattr(sys, "frozen", False):
        exec_location = sys.argv[0]
        # log.print_info("Identified Packaged Distribution at " + exec_location)
        abs_path = os.path.abspath(exec_location)
        # log.print_info("Absolute Path: " + abs_path)
        exec_dir = os.path.dirname(abs_path)
        # log.print_info(f"Executable located at :{exec_dir}")
    return exec_dir


def _add_default_config_value(config_path):
    """
    Add default value in a config path
    Args:
        config_path: Absolute config location
    """
    info = {
        "engine_name_1": {
            "hostname": "engine_name_1",
            "ip_address": "11.111.11.111",
            "username": "user",
            "password": "pass",
            "port": "80",
            "default": "true",
            "encrypted": "false",
            "use_https": "false",
        },
        "engine_name_2": {
            "hostname": "engine_name_2",
            "ip_address": "22.222.222.22",
            "username": "user",
            "password": "pass",
            "port": "80",
            "default": "true",
            "encrypted": "false",
            "use_https": "false",
        },
    }
    log.print_info(
        "Config full path is: {config_path} ".format(config_path=config_path)
    )
    with open(config_path, "w+") as out:
        json.dump(info, out, indent=4)


def format_err_msg(
        msg1,
        msg2=None,
        engine_name=None,
        object_name=None,
        err_code='ERR_DXI_GEN',
        operation=None,
        action='Rectify the error and re-try.'
):
    '''
    Function to format an error message to print for user.
    Format:
        error_code: msg1
        msg2
        Details: Engine - engine_name, Object - object_name, Operation - operation
        Possible Action: action
    Args:
        msg1: Error message string
        msg2: Error message descriptive string.
        engine_name: Name (hostname/ip) of the engine where error occurred.
        object_name: Name of the Delphix data object
        err_code: An error code if applicable
        operation: The delphix operation ( refresh, rewind etc) related to failure
        action: Possible resolution

    Returns:
        err_str: A formatted error message.
    '''
    e = '\t{}:{}\n\t{}\n\tDetails- Engine:{}, Object:{}, ' \
        'Operation:{}\n\tPossible Action: {}'.format(
        err_code,
        msg1,
        msg2,
        engine_name,
        object_name,
        operation,
        action
    )
    return e

def convert_string_to_datetime(tstr):
    """
    Converts timestamp in string format to datetime.
    Args:
        tstr (string): Timestamp string.
    Returns:
        ts_dt (datetime): Timestamp as a datetime object
    Raises:
        Exception: General Exception.
    """
    ts_dt = parser.parse(tstr)
    return ts_dt



#!/usr/bin/env python3
#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

"""
Link a MSSQL dSource
"""
from delphixpy.v1_10_2 import exceptions
from delphixpy.v1_10_2.web import database
from delphixpy.v1_10_2.web import environment
from delphixpy.v1_10_2.web import vo
from dxi._lib import dlpx_exceptions
from dxi._lib import get_references

from .dsource_link import DsourceLink


class DsourceLinkMssql(DsourceLink):
    """
    Derived class implementing linking of a MSSQL dSource
    """

    def __init__(
        self,
        dlpx_obj,
        dsource_name,
        db_passwd,
        db_user,
        dx_group,
        logsync,
        env_name,
        validated_sync_mode,
        initial_load_type,
        delphix_managed=True,
    ):
        """
        Constructor method
        :param dlpx_obj: A Delphix DDP session object
        :type dlpx_obj: lib.get_session.GetSession
        :param dsource_name: Name of the dsource
        :type dsource_name: str
        :param dx_group: Group name of where the dSource will reside
        :type dx_group: str
        :param db_passwd: Password of the db_user
        :type db_passwd: str
        :param db_user: Username of the dSource
        :type db_user: str
        :param db_type: dSource type. mssql, sybase or oracle
        :type db_type: str
        """
        super().__init__(dsource_name, db_passwd, db_user, dx_group, logsync, dlpx_obj, 'mssql')
        self.env_name = env_name
        self.validated_sync_mode = validated_sync_mode
        self.initial_load_type = initial_load_type
        self.delphix_managed = delphix_managed
        if delphix_managed:
            self.initial_load_type = "COPY_ONLY"

    # TODO delete below method in next release
    # def get_or_create_mssql_sourcecfg(
    #     self,
    #     env_name,
    #     db_install_path,
    #     stage_env,
    #     stage_instance,
    #     backup_path,
    #     backup_loc_passwd,
    #     backup_loc_user,
    #     backup_uuid=None,
    # ):
    #     """
    #     Create the sourceconfig used for provisioning an MSSQL dSource
    #     :param env_name: Name of the environment in Delphix
    #     :type env_name: str
    #     :param db_install_path: Path to where the Oracle binaries are installed
    #     :type db_install_path: str
    #     """
    #     env_obj = get_references.find_obj_by_name(
    #         self.dlpx_obj.server_session, environment, env_name
    #     )
    #     repo_ref = get_references.find_db_repo(
    #         self.dlpx_obj.server_session,
    #         "MSSqlInstance",
    #         env_obj.reference,
    #         db_install_path,
    #     )
    #
    #     # source config for single instance MSSQL
    #     sourcecfg_params = vo.MSSqlSIConfig()
    #     sourcecfg_params.user = self.db_user
    #     sourcecfg_params.credentials = vo.PasswordCredential()
    #     sourcecfg_params.credentials.password = self.db_passwd
    #     sourcecfg_params.database_name = self.dsource_name
    #     sourcecfg_params.repository = repo_ref
    #     sourcecfg_params.environment_user = env_obj.primary_user
    #     sourcecfg_params.recovery_model = "FULL"
    #     self.link_mssql_dsource(
    #         stage_env,
    #         stage_instance,
    #         backup_path,
    #         backup_loc_passwd,
    #         backup_loc_user,
    #         backup_uuid,
    #     )

    def link_mssql_dsource(
        self,
        source_env,
        stage_env,
        stage_instance,
        backup_path,
        backup_loc_passwd,
        backup_loc_user,
        uuid,
    ):
        """
        Link an MSSQL dSource
        :param stage_env: Name of the staging environment
        :type stage_env: str
        :param stage_instance: Name if the staging database instance
        :type stage_instance: str
        :param backup_path: Directory of where the backup is located
        :type backup_path: str
        :param backup_loc_passwd: Password of the shared backup path
        :type backup_loc_passwd: str
        :param backup_loc_user: Username for the shared backup path
        :type backup_loc_user: str
        """
        link_params = super().dsource_prepare_link(source_env_name=source_env)
        if self.delphix_managed:
            link_params.link_data.ingestion_strategy = (
                vo.DelphixManagedBackupIngestionStrategy()
            )
            link_params.link_data.ingestion_strategy.backup_policy = "PRIMARY"
            link_params.link_data.ingestion_strategy.compression_enabled = (
                False
            )
        else:
            link_params.link_data.ingestion_strategy = (
                vo.ExternalBackupIngestionStrategy()
            )
            link_params.link_data.ingestion_strategy.validated_sync_mode = (
                self.validated_sync_mode
            )
        link_params.link_data.sourcing_policy = vo.SourcingPolicy()
        link_params.link_data.sourcing_policy.logsync_enabled = False
        if (
            self.validated_sync_mode
            and self.validated_sync_mode == "TRANSACTION_LOG"
        ):
            link_params.link_data.sourcing_policy.logsync_enabled = (
                self.logsync
            )
        try:
            env_obj_ref = get_references.find_obj_by_name(
                self.dlpx_obj.server_session, environment, stage_env
            ).reference
            ppt_repo_ref = get_references.find_db_repo(
                self.dlpx_obj.server_session,
                "MSSqlInstance",
                env_obj_ref,
                stage_instance,
            )
            link_params.link_data.ppt_repository = ppt_repo_ref
        except dlpx_exceptions.DlpxException as err:
            raise dlpx_exceptions.DlpxException(
                f"Could not link {self.dsource_name}:\n{err}"
            )
        # specifying backup locations
        link_params.link_data.shared_backup_locations = []
        if backup_path and backup_path != "auto":
            link_params.link_data.shared_backup_locations = backup_path.split(
                ":"
            )
        if backup_loc_passwd:
            link_params.link_data.backup_location_credentials = (
                vo.PasswordCredential()
            )
            link_params.link_data.backup_location_credentials.password = (
                backup_loc_passwd
            )
            link_params.link_data.backup_location_user = backup_loc_user
        # specify the initial sync Parameters
        if self.initial_load_type and self.initial_load_type == "SPECIFIC":
            link_params.link_data.sync_parameters = (
                vo.MSSqlExistingSpecificBackupSyncParameters()
            )
            link_params.link_data.sync_parameters.backup_uuid = uuid
        elif self.initial_load_type and self.initial_load_type == "COPY_ONLY":
            link_params.link_data.sync_parameters = (
                vo.MSSqlNewCopyOnlyFullBackupSyncParameters()
            )
            link_params.link_data.sync_parameters.backup_policy = "PRIMARY"
            link_params.link_data.sync_parameters.compression_enabled = False
        else:
            link_params.link_data.sync_parameters = (
                vo.MSSqlExistingMostRecentBackupSyncParameters()
            )

        try:
            database.link(self.dlpx_obj.server_session, link_params)
        except (
            exceptions.HttpError,
            exceptions.RequestError,
            exceptions.JobError,
        ) as err:
            dlpx_exceptions.DlpxException(
                f"Database link failed for {self.dsource_name}:{err}"
            )




#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

import click
from dxi._lib.util import boolean_based_system_exit
from dxi.container.dxi_container import DXIContainer
from dxi.container.dxi_container import DXIContainerConstants


@click.group(
    short_help="\b\nPerform Delphix Self-Service Container Operations.\n"
               "[create | refresh | reset | restore | delete | "
               "add-owner \nremove-owner | connection-info | enable | list]"
)
def container():
    """
    Self-Service Container operations
    """
    pass


# List Command
@click.option(
    "--engine",
    default=DXIContainerConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXIContainerConstants.SINGLE_THREAD),
    default=DXIContainerConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file with filename.",
    default=DXIContainerConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path to the logs folder.",
    default=DXIContainerConstants.LOG_FILE_PATH,
)
@click.option(
    "--poll",
    help="The number of seconds to wait between job polls.",
    default=DXIContainerConstants.POLL,
)
@container.command()
def list(engine, single_thread, config, log_file_path, poll):
    """
    List all self-service containers on a given engine
    """
    ss_container = DXIContainer(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(ss_container.list())


# Create Container Command
@container.command()
@click.option(
    "--container-name",
    required=True,
    help="Name of the self-service container.",
)
@click.option(
    "--db-name",
    required=True,
    help="\b\nName of the datasets to create the self-service container.\n"
    "If the container should contain multiple datasets,\n"
    "separate the names with colon (:).\n"
    "Sample: db1:db2:db3",
)
@click.option(
    "--template-name",
    required=True,
    help="Name of the parent self-service template for the container.",
)
@click.option(
    "--engine",
    default=DXIContainerConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file with filename.",
    default=DXIContainerConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path to the logs folder.",
    default=DXIContainerConstants.LOG_FILE_PATH,
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXIContainerConstants.SINGLE_THREAD),
    default=DXIContainerConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIContainerConstants.POLL),
    default=DXIContainerConstants.POLL,
)
def create(
    container_name,
    template_name,
    db_name,
    engine,
    single_thread,
    config,
    log_file_path,
    poll,
):
    """
    Create a self-service container.
    """
    ss_container = DXIContainer(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(
        ss_container.create(container_name, template_name, db_name=db_name)
    )


# Delete Container Command
@click.option(
    "--container-name",
    required=True,
    help="Name of the self-service container.",
)
@click.option(
    "--keep-vdbs",
    help="\b\nIf set, deleting the container will not remove \n"
    "the underlying virtual dataset(s)\n"
    "[Default: False]",
    default=False,
    is_flag=True,
)
@click.option(
    "--engine",
    default=DXIContainerConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file with filename.",
    default=DXIContainerConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path to the logs folder.",
    default=DXIContainerConstants.LOG_FILE_PATH,
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXIContainerConstants.SINGLE_THREAD),
    default=DXIContainerConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIContainerConstants.POLL),
    default=DXIContainerConstants.POLL,
)
@container.command()
def delete(
    container_name,
    keep_vdbs,
    engine,
    single_thread,
    config,
    log_file_path,
    poll,
):
    """
    Delete a self-service container
    """
    ss_container = DXIContainer(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(
        ss_container.delete(container_name, keep_vdbs=keep_vdbs)
    )


# Reset Container Command
@container.command()
@click.option(
    "--container-name",
    required=True,
    help="Name of the self-service container.",
)
@click.option(
    "--engine",
    default=DXIContainerConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file with filename.",
    default=DXIContainerConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path to the logs folder.",
    default=DXIContainerConstants.LOG_FILE_PATH,
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXIContainerConstants.SINGLE_THREAD),
    default=DXIContainerConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIContainerConstants.POLL),
    default=DXIContainerConstants.POLL,
)
def reset(container_name, engine, single_thread, config, log_file_path, poll):
    """
    Undo the last refresh or restore operation on a self-service container.
    """
    ss_container = DXIContainer(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(ss_container.reset(container_name))


# Refresh Container Command
@container.command()
@click.option(
    "--container_name",
    required=True,
    help="Name of the self-service container.",
)
@click.option(
    "--engine",
    default=DXIContainerConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file with filename.",
    default=DXIContainerConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path to the logs folder.",
    default=DXIContainerConstants.LOG_FILE_PATH,
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXIContainerConstants.SINGLE_THREAD),
    default=DXIContainerConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIContainerConstants.POLL),
    default=DXIContainerConstants.POLL,
)
def refresh(
    container_name, engine, single_thread, config, log_file_path, poll
):
    """
    Refresh a self-service container.
    """
    ss_container = DXIContainer(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(ss_container.refresh(container_name))


# Restore Container Command
@container.command()
@click.option(
    "--bookmark-name",
    required=True,
    help="Name of the self-service bookmark to restore the container",
)
@click.option(
    "--container-name",
    required=True,
    help="Name of the self-service container.",
)
@click.option(
    "--engine",
    default=DXIContainerConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file with filename.",
    default=DXIContainerConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path to the logs folder.",
    default=DXIContainerConstants.LOG_FILE_PATH,
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXIContainerConstants.SINGLE_THREAD),
    default=DXIContainerConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIContainerConstants.POLL),
    default=DXIContainerConstants.POLL,
)
def restore(
    container_name,
    bookmark_name,
    engine,
    single_thread,
    config,
    log_file_path,
    poll,
):
    """
    Restore a self-service container to a specific self-service bookmark.
    """
    ss_container = DXIContainer(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(
        ss_container.restore(container_name, bookmark_name)
    )


# Lists hierarchy Command
@click.option(
    "--container-name",
    required=True,
    help="Name of the self-service container.",
)
@click.option(
    "--engine",
    default=DXIContainerConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file with filename.",
    default=DXIContainerConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path to the logs folder.",
    default=DXIContainerConstants.LOG_FILE_PATH,
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXIContainerConstants.SINGLE_THREAD),
    default=DXIContainerConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIContainerConstants.POLL),
    default=DXIContainerConstants.POLL,
)
@container.command()
def connection_info(
    container_name, engine, single_thread, config, log_file_path, poll
):
    """
    List connection information for a self-service container.
    """
    ss_container = DXIContainer(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(ss_container.connection_info(container_name))


# Add owner Command
@container.command()
@click.option(
    "--container-name",
    required=True,
    help="Name of the self-service container.",
)
@click.option(
    "--owner-name",
    required=True,
    help="Name of the owner user for the container",
)
@click.option(
    "--engine",
    default=DXIContainerConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file with filename.",
    default=DXIContainerConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path to the logs folder.",
    default=DXIContainerConstants.LOG_FILE_PATH,
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXIContainerConstants.SINGLE_THREAD),
    default=DXIContainerConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIContainerConstants.POLL),
    default=DXIContainerConstants.POLL,
)
def add_owner(
    owner_name,
    container_name,
    engine,
    single_thread,
    config,
    log_file_path,
    poll,
):
    """
    Add an owner to a self-service container.
    """
    # print(owner_name, container_name)
    ss_container = DXIContainer(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(
        ss_container.add_owner(container_name, owner_name)
    )


# Delete owner Command
@container.command()
@click.option(
    "--container-name",
    required=True,
    help="Name of the self-service container.",
)
@click.option(
    "--owner-name",
    required=True,
    help="Name of the owner user for the container",
)
@click.option(
    "--engine",
    default=DXIContainerConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file with filename.",
    default=DXIContainerConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path to the logs folder.",
    default=DXIContainerConstants.LOG_FILE_PATH,
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(DXIContainerConstants.SINGLE_THREAD),
    default=DXIContainerConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(DXIContainerConstants.POLL),
    default=DXIContainerConstants.POLL,
)
def remove_owner(
    container_name,
    owner_name,
    engine,
    single_thread,
    config,
    log_file_path,
    poll,
):
    """
    Remove an owner from a self-service container.
    """
    ss_container = DXIContainer(
        engine=engine,
        single_thread=single_thread,
        config=config,
        log_file_path=log_file_path,
        poll=poll,
    )
    boolean_based_system_exit(
        ss_container.remove_owner(container_name, owner_name)
    )



#!/usr/bin/env python3
#
# Copyright (c) 2021 by Delphix. All rights reserved.
#
"""
Link an ASE Sybase dSource
"""
from delphixpy.v1_10_2 import exceptions
from delphixpy.v1_10_2.web import database
from delphixpy.v1_10_2.web import environment
from delphixpy.v1_10_2.web import repository
from delphixpy.v1_10_2.web import vo
from dxi._lib import dlpx_exceptions
from dxi._lib import get_references

from .dsource_link import DsourceLink


class DsourceLinkASE(DsourceLink):
    """
    Derived class implementing linking of a ASE Sybase dSource
    """

    def __init__(
        self,
        dlpx_obj,
        dsource_name,
        db_passwd,
        db_user,
        dx_group,
        logsync,
        stage_repo,
    ):
        """
        Constructor method
        :param dlpx_obj: A Delphix DDP session object
        :type dlpx_obj: lib.get_session.GetSession
        :param dsource_name: Name of the dsource
        :type dsource_name: str
        :param dx_group: Group name of where the dSource will reside
        :type dx_group: str
        :param db_passwd: Password of the db_user
        :type db_passwd: str
        :param db_user: Username of the dSource
        :type db_user: str
        :param logsync: Enable logsync
        :type logsync: bool
        :param db_type: dSource type. mssql, sybase or oracle
        :type db_type: str
        """
        super().__init__(dsource_name, db_passwd, db_user, dx_group, logsync, dlpx_obj, 'sybase')
        self.stage_repo = stage_repo

    def link_ase_dsource(self, backup_path, bck_file, create_bckup, env_name, source_env):
        """
        Link an ASE dSource
        :param backup_path: Path to the ASE/MSSQL backups
        :type backup_path: str
        :param bck_file: Fully qualified name of backup file
        :type bck_file: str
        :param create_bckup: Create and ingest a new Sybase backup
        :type create_bckup: str
        :param env_name: Name of the environment where the dSource running
        :type env_name: str
        """
        link_params = super().dsource_prepare_link()
        link_params.link_data.load_backup_path = backup_path
        if bck_file:
            link_params.link_data.sync_parameters = (
                vo.ASESpecificBackupSyncParameters()
            )
            bck_files = bck_file.split(" ")
            link_params.link_data.sync_parameters.backup_files = bck_files
        elif create_bckup:
            link_params.link_data.sync_parameters = (
                vo.ASENewBackupSyncParameters()
            )
        else:
            link_params.link_data.sync_parameters = (
                vo.ASELatestBackupSyncParameters()
            )
        try:

            env_user_ref_staging = (
                link_params.link_data.stage_user
            ) = get_references.find_obj_by_name(
                self.dlpx_obj.server_session, environment, env_name
            ).primary_user
            env_ref = get_references.find_obj_by_name(
                self.dlpx_obj.server_session, environment, env_name)
            link_params.link_data.staging_host_user = env_user_ref_staging

            env_user_ref_source =  get_references.find_obj_by_name(
                self.dlpx_obj.server_session, environment, source_env
            ).primary_user

            link_params.link_data.source_host_user = env_user_ref_source
            link_params.link_data.staging_repository = get_references.find_obj_by_name(  # noqa
                self.dlpx_obj.server_session, repository, self.stage_repo, env_name=env_ref.reference
            ).reference

        except dlpx_exceptions.DlpxException as err:
            raise dlpx_exceptions.DlpxException(
                f"Could not link {self.dsource_name}:\n{err}"
            )
        try:
            dsource_ref = database.link(
                self.dlpx_obj.server_session, link_params
            )
            print(f"{dsource_ref} successfully linked {self.dsource_name}")
        except (exceptions.RequestError, exceptions.HttpError) as err:
            raise dlpx_exceptions.DlpxException(
                f"Database link failed for {self.dsource_name}:\n{err}"
            )
